Widget["Frame", {

(* Load Package *)
Script[ Needs["Tetris`"]; ],


"title" -> "Mathematica Tetris", 


(* Timer default *)
Widget["Timer", {"delay" -> Script[$refreshTiming],
	BindEvent["action", Script[ paintPieceRoutine["down", "engine", 0, 1]; ]]
}, Name -> "moveTimer"],


(* Define Actions *)
Widget["Action", {
	"name" -> "New game",
    "shortDescription" -> "New game",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"N"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ newGame[]; clearScreen[]; setNextPiecePanel[]; InvokeMethod[{"moveTimer", "start"}];]]
}, Name -> "newAction"],


Widget["Action", {
	"name" -> "Show all pieces...",
    "shortDescription" -> "Show all pieces...",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"S"}],
	BindEvent["action",
		Script[ showAllPieces[];]]
}, Name -> "showAllPiecesAction"],

Widget["Action", {
	"name" -> "Exit",
    "shortDescription" -> "Exit",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"X"}],
	BindEvent["action",
		Script[ GameOver[]; CloseGUIObject[];]]
}, Name -> "exitAction"],

Widget["Action", {
	"name" -> "Left",
    "shortDescription" -> "Left",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"L"}],
	BindEvent["action",
		Script[ paintPieceRoutine["left", "user", 1, 0];]]
}, Name -> "leftAction"],

Widget["Action", {
	"name" -> "Right",
    "shortDescription" -> "Right",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"J"}],
	BindEvent["action",
		Script[ paintPieceRoutine["right", "user", -1, 0];]]
}, Name -> "rightAction"],

Widget["Action", {
	"name" -> "Rotate",
    "shortDescription" -> "Rotate",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"I"}],
	BindEvent["action",
		Script[ 
			paintRotatedPieceRoutine[$pieceOriginX, $pieceOriginY, $pieceOrientation, getNextRotation[$pieceOrientation]]; ]]
}, Name -> "rotateAction"],

Widget["Action", {
	"name" -> "Drop",
    "shortDescription" -> "Drop",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"K"}],
	BindEvent["action",
		Script[ paintPieceRoutine["drop", "user", 0, 1];]]
}, Name -> "dropAction"],

Widget["Action", {
	"name" -> "Pause",
    "shortDescription" -> "Pause",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"P"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Pause16.gif"}],
	BindEvent["action",
		Script[ InvokeMethod[{"moveTimer", "stop"}];]]
}, Name -> "pauseAction"],

Widget["Action", {
	"name" -> "Continue",
    "shortDescription" -> "Continue",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"C"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ InvokeMethod[{"moveTimer", "start"}];]]
}, Name -> "continueAction"],

Widget["Action", {
	"name" -> "Reset board",
    "shortDescription" -> "Reset board",
(*	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"C"}], *)
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Stop16.gif"}],
	BindEvent["action",
		Script[ clearBoard[]; clearScreen[]; ]]
}, Name -> "clearScreenAction"],

Widget["Action", {
	"name" -> "Piece 1",
    "shortDescription" -> "Piece 1",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"1"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[1]; paintPieceRoutine["paint", "user", 0, 0]; ]]
}, Name -> "testPiece1Action"],

Widget["Action", {
	"name" -> "Piece 2",
    "shortDescription" -> "Piece 2",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"2"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[2]; paintPieceRoutine["paint", "user", 0, 0]; ]]
}, Name -> "testPiece2Action"],

Widget["Action", {
	"name" -> "Piece 3",
    "shortDescription" -> "Piece 3",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"3"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[3]; paintPieceRoutine["paint", "user", 0, 1]; ]]
}, Name -> "testPiece3Action"],

Widget["Action", {
	"name" -> "Piece 4",
    "shortDescription" -> "Piece 4",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"4"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[4]; paintPieceRoutine["paint", "user", 0, 1]; ]]
}, Name -> "testPiece4Action"],

Widget["Action", {
	"name" -> "Piece 5",
    "shortDescription" -> "Piece 5",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"5"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[5]; paintPieceRoutine["paint", "user", 0, 0]; ]]
}, Name -> "testPiece5Action"],

Widget["Action", {
	"name" -> "Piece 6",
    "shortDescription" -> "Piece 6",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"6"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[6]; paintPieceRoutine["paint", "user", 0, 0]; ]]
}, Name -> "testPiece6Action"],

Widget["Action", {
	"name" -> "Piece 7",
    "shortDescription" -> "Piece 7",
	"accelerator" -> Widget["MenuShortcut", InitialArguments -> {"7"}],
	"icon" -> Widget["Icon", {"path" -> "Tetris/Images/Play16.gif"}],
	BindEvent["action",
		Script[ defaultSettings[7]; paintPieceRoutine["paint", "user", 0, 0]; ]]
}, Name -> "testPiece7Action"],


(* Set up menus *)
"menus" -> Widget["Tetris/Menus/TetrisMenu"],


(* Main display *)
Widget["Panel", {
	WidgetGroup[{

		WidgetGroup[{
			(* Main Board *)
			WidgetGroup[{
		    Widget["IndexedImagePanel", { 
		    	"preferredSize" -> 
		        	Widget["Dimension", {"width" -> Script[$boardWidth*$pixelSize], "height" -> Script[$boardHeight*$pixelSize]}], 
		        "imageWidth" -> Script[$boardWidth],
		        "imageHeight" -> Script[$boardHeight],
		        "imagePixelSize" -> Script[$pixelSize],
		        "imageColorMapSize" -> Script[Length[colorSet[]] ],
		        "imageColorComponents" ->  Script[Flatten[{colorSet[] }] ],
				"imagePixels" -> Script[Flatten[ Table[0, {$boardWidth}, {$boardHeight}]] ]}, Name -> "gamePanel"]
			}, WidgetLayout -> {"Border" -> {{RGBColor[0.5, 0.5, 0.5], 1}, {{5, 5}, {5, 5}}} }]

		}],
		WidgetSpace[10],
		WidgetGroup[{
			WidgetGroup[{
			Widget["IndexedImagePanel", {
				"preferredSize" -> 
					Widget["Dimension", {"width" -> 5*15, "height" -> 3*15}],
				"imageWidth" -> 5,
				"imageHeight" -> 3,
				"imagePixelSize" -> 15,
				"imageColorMapSize" -> 8,
				"imageColorComponents" -> Script[Flatten[{colorSet[] }] ],
				"imagePixels" -> Script[Flatten[ Table[0, {5}, {3}]] ]
				}, Name -> "nextPiecePanel"]
			}, WidgetLayout -> { "Border" -> "Next Piece" }],
			WidgetFill[],
			Widget["Panel", {
				Widget["Label",  {"text"->"Speed:"}]
			}],
			Widget["Slider", {
				"minimum" -> 50, 
				"maximum" -> 750,
				"value" -> 250,
				BindEvent["change",
					Script[ 
						SetPropertyValue[{"moveTimer", "delay"},
							PropertyValue[{"slider", "value"}] ]]]},
				Name -> "slider"],
			WidgetFill[],
			Widget["Panel", {
				Widget["Label",  {"text"->"Statistics:"}]
			}],

			(* Tabbed Content Panes *)
			Widget["ScrollPane", {
				"preferredSize" -> Widget["Dimension", {"width" -> 200, "height" -> 150}],
				"viewportView" -> 
					Widget["TextPanel", {
						"preferredSize" -> Widget["Dimension", {"width" -> 200, "height" -> 200}],
						"text" -> ""},
					Name -> "statusArea"]
			}, WidgetLayout -> {"Stretching" -> { False, False}},
			Name -> "statusPane"]
		}]

	}, WidgetLayout -> { "Stretching" -> { False, False}}]
}, WidgetLayout -> {"Border" -> {10}, "Stretching" -> { False, False}},
	Name -> "mainPanel"],



Script[
(*	$boardWidth = 10;
	$boardHeight = 20;
	$pixelSize = 15;
*)
	$pieceOriginX = 4;
	$pieceOriginY = 1;
	$pieceStyle = 3;
	$pieceOrientation = 1;
],

Script[{}, ScriptSource -> "InterfaceCode.m"],


(* Set and draw grid *)
SetPropertyValue[{"gamePanel", "imageGrid"}, True],
Script[
	SetPropertyValue[{"gamePanel", "imageGridColor"}, 
	Widget["Color", InitialArguments -> {120, 120, 120}]];
],
InvokeMethod[{"gamePanel", "repaint"}],

SetPropertyValue[{"nextPiecePanel", "imageGrid"}, True],
Script[
	SetPropertyValue[{"nextPiecePanel", "imageGridColor"}, 
	Widget["Color", InitialArguments -> {120, 120, 120}]];
],
InvokeMethod[{"nextPiecePanel", "repaint"}]


}, Name -> "BasicBoard"]