Needs["Tetris`"];


(* Paint piece *)
paintPieceRoutine[action_, user_, h_, v_, engine___]:=
	Module[{ht = $pieceOriginX + h, vt = $pieceOriginY + v, lst, tempBoard},

		(* Remove current piece from temporary board *)
		tempBoard = $TetrisBoard;
		lst = getPieceCoordinates[$pieceOriginX, $pieceOriginY, $pieceStyle, $pieceOrientation];
		If[$debug === True, Print["paintPieceRoutine: ", lst]];
		(Part[tempBoard, #[[1]]+1, #[[2]]+1 ] = 0) & /@ lst;

		If[$debug === True, Print["paintPieceRoutine: ", tempBoard]];

		If[	PiecePlacementQ[tempBoard, ht, vt, $pieceStyle, $pieceOrientation],
			(
			If[$debug === True, Print["Paint piece"]];
			paintPiece[$pieceOriginX, $pieceOriginY, $pieceStyle, $pieceOrientation, 0, False];

			paintPiece[ht, vt, $pieceStyle, $pieceOrientation, $pieceStyle, True];
			$pieceOriginX = ht;
			$pieceOriginY = vt;
(*			writeStatusString[ StringJoin["{", ToString[ht], ", ", ToString[vt], "}, ", 
				ToString[h], ", ", ToString[v], ", ", ToString[$pieceStyle]] ]; *)
			)
		,
			(
			If[(gameOverQ[] === True) && ($ActiveGameQ === True), 
				(
					InvokeMethod[{"moveTimer", "stop"}];
					$ActiveGameQ = False;
					writeStatusString["Game Over"];
					writeStatusString["-------------"];
					writeStatusString["Seconds: " <> ToString[ ElapsedTime[StartTime, AbsoluteTime[]] ] ];
					writeStatusString["Pieces: " <> ToString[PieceCount] ];
					writeStatusString["Rows: " <> ToString[RowCount] ];
				)];
			If[$debug === True, Print[action, ", ", user]];
			If[$debug === True, Print[$previousAction, ", ", $previousUser]];
			If[((action === $previousAction) && (user === "engine") && 
				($previousUser === "engine") && ($ActiveGameQ === True))
			,
				$previousUser = user;
				$previousAction = action;
				CollapseRows[];
				AddPieceToQueue[];
			,
				$previousUser = user;
				$previousAction = action;
			];
			If[$debug === True, Print["Out of bounds"]];)
	]];


(* Paint rotated piece *)
paintRotatedPieceRoutine[x_, y_, rot_, newRot_]:=
	Module[{tempBoard, lst},
		tempBoard = $TetrisBoard;
		lst = getPieceCoordinates[x, y, $pieceStyle, $pieceOrientation];
		(Part[tempBoard, #[[1]]+1, #[[2]]+1 ] = 0) & /@ lst;
		If[	PiecePlacementQ[tempBoard, x, y, $pieceStyle, newRot],
			(
			If[$debug === True, Print["Paint rotated piece"]];
			paintPiece[x, y, $pieceStyle, $pieceOrientation, 0, False];
			$pieceOrientation = newRot;
			paintPiece[x, y, $pieceStyle, $pieceOrientation, $pieceStyle, True];
(*			writeStatusString[ StringJoin["{", ToString[ht], ", ", ToString[vt], "}, ", 
				ToString[h], ", ", ToString[v], ", ", ToString[$pieceStyle]] ]; *)
			)
		,
			If[$debug === True, Print["Out of bounds"]];
	]];

(* Paint a piece *)
paintPiece[x_, y_, piece_, orientation_, color_, paint_]:=
	Module[{lst},
		lst = getPieceCoordinates[x, y, piece, orientation];
		InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
			getPieceCoordinates[x, y, piece, orientation], 
			color, True, InvokeThread -> "Dispatch"];
		(If[$debug === True, Print["paintPiece: ", Part[$TetrisBoard, #[[1]]+1, #[[2]]+1 ]]];
		Part[$TetrisBoard, #[[1]]+1, #[[2]]+1 ] = color) & /@ lst;
	];


(* Write status string *)
writeStatusString[s_]:=
	Module[{str},
 		If[$statusMode === True, 
			str = PropertyValue[{"statusArea", "text"} ];
			If[str === "", str = s <> "\n", 
				str = str <> s <> "\n"];
			SetPropertyValue[{"statusArea", "text"}, str ];
		]];


showAllPieces[]:=
	(
	(* Stop timer, if running *)
	InvokeMethod[{"moveTimer", "stop"}];

	(* Color board black *)
	clearScreen[];

	(* Paint all pieces *)
		(* 'O' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[5, 1, 1, 1], 1, False];
		(* 'I' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[5, 4, 2, 1], 2, False];
		(* 'S' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[3, 7, 3, 1], 3, False];
		(* 'Z' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[6, 10, 4, 1], 4, False];
		(* 'L' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[2, 12, 5, 2], 5, False];
		(* 'J' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[7, 14, 6, 4], 6, False];
		(* 'T' *)
			InvokeMethod[{"gamePanel", "fillImagePixelArray"}, 
				getPieceCoordinates[4, 17, 7, 3], 7, False];

		InvokeMethod[{"gamePanel", "repaint"}];
	);


(* Clear screen *)
clearScreen[]:=
	(SetPropertyValue[{"gamePanel", "imagePixels"},
		Flatten[ Table[0, {$boardWidth}, {$boardHeight}]]];
	SetPropertyValue[{"statusArea", "text"}, "" ];);

