(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    224509,       3058]*)
(*NotebookOutlinePosition[    227292,       3142]*)
(*  CellTagsIndexPosition[    227218,       3136]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Tetris User Guide", "Section",
  CellTags->"Tetris"],

Cell[TextData[{
  "Tetris is an Add-on to Mathematica which allows the user to play Tetris as \
well as plot the current game play. The GUI is made using ",
  ButtonBox["GUIKit",
    ButtonData:>{
      URL[ "http://www.wolfram.com/solutions/mathlink/guikit/"], None},
    ButtonStyle->"Hyperlink"],
  " the user-interface development tool for ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"Tetris"],

Cell[TextData[{
  "Please send comments/questions to Andy Hunt (",
  ButtonBox["andy@wolfram.com",
    ButtonData:>{
      URL[ "mailto:andy@wolfram.com"], None},
    ButtonStyle->"Hyperlink"],
  ")."
}], "Text",
  CellTags->"Tetris"],

Cell[CellGroupData[{

Cell["Running Tetris", "Subsection",
  CellTags->"Tetris"],

Cell["Loading package", "MathCaption",
  CellTags->"Tetris"],

Cell[BoxData[
    \(Needs["\<Tetris`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"Tetris"],

Cell["Start GUI", "MathCaption",
  CellTags->"Tetris"],

Cell[BoxData[
    \(\(Tetris[];\)\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->"Tetris"],

Cell["Sample screen", "MathCaption",
  CellTags->"Tetris"],

Cell[BoxData[
    \(gui = TetrisGUI[]; \)], "Input",
  CellLabel->"In[3]:=",
  CellTags->"Tetris"],

Cell["[play for a while...]", "IndentedText",
  CellTags->"Tetris"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GUIScreenShot[gui] // Show\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->"Tetris"],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000680001NB000`400?l00000o`00003oo`3/k>aj
0>c/k0030=WIf@3oool0oooo00`0oooo003o0>c/k7X0k>c/00h0fMWI0?ooo`3oool0f][J08N7Q`3Y
jNT0oooo0=[JfP27QhL0jNWY0?ooo`3Jf]X0QhN70>WYj@40oooo003o0?3`l7X0l?3`00h0fMWI0?oo
o`3oool0oooo0=[JfP27QhL0jNWY0?ooo`3Jf]X0QhN70>WYj@3oool0f][J08N7Q`40oooo003o0?3`
l7X0l?3`00<0fMWI0?ooo`3oool00P3oool02@3EeMD0QhN70>WYj@3oool0f][J08N7Q`3YjNT0oooo
0=[JfP010?ooo`00o`3/k>aj0>c/k0030=WIf@3oool0oooo00<0oooo00L0emOG08N7Q`3YjNT0oooo
0=[JfP27QhL0jNWY0080oooo003o0>c/k7X0k>c/00<0fMWI0?ooo`3oool0103oool01`3GemL0QhN7
0>WYj@3oool0f][J08N7Q`3YjNT00@3oool00?l0l?3`NP3`l?000`3IfMT0oooo0?ooo`050?ooo`06
0=OGe`27QhL0jNWY0?ooo`3Lg=`0QhN70@3oool00?l0l?3`NP3`l?000`3IfMT0oooo0?ooo`060?oo
o`050=WIf@27QhL0jNWY0?ooo`3Lg=`00@3oool00?l0k>c/NP3/k>`00`3IfMT0oooo0?ooo`070?oo
o`030=_Kf`27QhL0jNWY0080oooo003o0>c/k7X0k>c/00<0fMWI0?ooo`3oool0203oool00`3Kfm/0
QhN70>WYj@010?ooo`002P3`l?2R07moO`X0l?3`00<0^KVi0<o?c`3MgMd0`03MgMd00`3IfMT0oooo
0?ooo`090?ooo`40g=cL0@27QhL10?ooo`002P3`l?2R07moO`X0l?3`00<0`l?30?3`l03no_h0`03n
o_h00`3IfMT0oooo0?ooo`0:0?ooo`40g=cL0@3oool000X0k>c/00<0Ogmo0>c/k03/k>`0W@3/k>`2
07moO`X0k>c/00<0`l?30?3`l03oool0`03oool00`3IfMT0oooo0?ooo`0<0?ooo`002P3/k>`00`1o
Ogl0k>c/0>c/k02M0>c/k080Ogmo2P3/k>`00`33`l<0l?3`0?ooo`300?ooo`030=WIf@3oool0oooo
00`0oooo000:0?3`l00307moO`3`l?00l?3`09d0l?3`0P1oOgl:0?3`l0030<?3``3`l?00oooo0<00
oooo3`3IfMT000X0l?3`00<0Ogmo0?3`l03`l?00W@3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0
``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/
k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=08;oO080N7Qh3@22og`2
07QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/2
07QhN0d0nodk00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool0
0`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN00000000000
00/000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207Qh
N0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0?3`l03`l?000P3`l?0207mo
O`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0
P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0
nodk00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00
`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/00000
0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm6
0P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/
00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k00307QhN0000000000000/000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1h
N7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0
N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/
k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=
08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=
0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?3
0?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN0000000000000/000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;o
O080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0?3`
l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=08;oO080
N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80
N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03o
ool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307Qh
N0000000000000/000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh
3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0>c/k03/k>`0
0P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=08;oO080N7Qh3@22
og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3k
oC/207QhN0d0nodk00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3o
ool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000
000000/000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH2
07QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0?3`l03`l?000P3`l?02
07moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1o
Ogl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=08;oO080N7Qh3@22og`207Qh
N0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207Qh
N0d0nodk00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`
l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/0
00000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d0
oCm60P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0
k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`
0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?3
0?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`00<0
`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k003
0>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=
00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh
0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k009
0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?do
AP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081n
o@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`
l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l003
07QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080
N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`l03`
l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l000
2P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh
3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh
3@3koC/207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0
``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m
?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m
?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/
k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH2
07QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/2
07QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool0
0`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0oCm6
00/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207Qh
N0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207mo
O`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0
oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0
000000<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00
`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm6
0P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km
0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/
00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1h
N7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0
N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/
k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1hN7P=
0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=
081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?3
0?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0000
0080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`
l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=0?doAP80
N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80
N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03o
ool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh
103/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3o
ool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0
oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH2
07QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0?3`l03`l?000P3`l?02
07moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1o
Ogl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207Qh
N0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207Qh
N0d0P7km00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`
l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0
oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0
P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0
k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`
0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d00000
0P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km
00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?3
0?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1h
N7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1h
N7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0
`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k003
0>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=
0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh
0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k009
0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=07co
n`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081n
o@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`
l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l003
07QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80
N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0?3`l03`
l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l000
2P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh
3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh
3@20O_d207QhN0d0P7km00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0
``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03m
?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m
?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0>c/k03/k>`00P3/
k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH2
07QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d2
07QhN0d0P7km00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool0
0`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03m?dH0oCm6
00/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?elo@80N7Qh3@3m?dH207Qh
N0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0P7km00<0N7Qh0?3`l03`l?000P3`l?0207mo
O`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l00307QhN03m?dH0oCm600/0oCm60P1hN7P=07con`80N7Qh3@3m?dH207QhN0d0
00000P1hN7P=0?elo@80N7Qh3@3m?dH207QhN0d0P7km0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0
P7km00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00
`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/k>`207moO`X0k>c/
00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k9H0N7Qh103/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/
k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=
07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=
081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?3
0?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m
>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`
l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80
N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80
N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03o
ool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307Qh
N03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh
3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`0
0P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1l
oo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00
000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3o
ool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03mO?d0
oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d2
07QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?02
07moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1o
Ogl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207Qh
N0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207Qh
N0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`
l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0
oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0
P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0
k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`
0?3`l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk
0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml
00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?3
0?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1h
N7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1h
N7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0
`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k003
0>c/k00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=
0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh
0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k009
0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=07co
n`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081n
o@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`
l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l003
07QhN03mO?d0oGcm00/0oGcm0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0nodk0P1hN7P=0?_m>`80
N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`
l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l000
2P3`l?000`1oOgl0l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`00<0`l?30?3`l03oool0
``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/
k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=0?elo@80N7Qh3@000002
07QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1hN7P=07con`80N7Qh3@1loo/2
07QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool0
0`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03mO?d0oGcm
00/0oGcm0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207Qh
N0d0O?ok0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207mo
O`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0
nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0
P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00
`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm
0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok
0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/
00<0`l?30?3`l03oool02P3oool00`3/k>`0kno_0?ooo`0>0?ooo`80l_;bYP3oool00`3`l?00`l?3
0>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1h
N7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1h
N7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00D0
`l?30?3`l03oool0=SHf07MgM`020?ooo`0:0?KfmP0V9RH0AdM70?knoP3Kfm/0=CDe00l?3`0<30`0
>CTi0=cLg080oooo00<0i^KV00410@27QhL00P3oool00`19BDT09bLW0?ooo`020?ooo`0502DU9@0@
4100820P04a<C03fm_H00P3oool00`1]KFd0DU9B0?ooo`050?ooo`0301@D50000000000000800000
00<0E5AD0?ooo`3oool0TP3oool00`3`l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`
l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1h
N7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0
N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00D0`l?30?3`l03oool0=SHf07MgM`020?ooo`0D06m_
K`0E5AD0inOW0?knoP0c<c<0Fe]K0?7al@3_knl0EUIF03De=@3no_h0oooo09jNWP0?3`l0>3Ph0?oo
o`3hn?P02PX:00l?3`3Cdm<20?ooo`050<71`@3kno/0o_kn03Hf=P2BTY800P3oool00`3Kfm/0e=CD
0?ooo`050?ooo`0305EEE@0^;Rh0i^KV0080i^KV00<0kno_0?ooo`3oool0TP3oool00`3`l?00`l?3
0?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03mO?d0oGcm00/0oGcm0P1h
N7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1h
N7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00d0
`l?30?3`l03oool0=SHf07MgM`3oool0`<3000820P2c/k<0oooo0=SHf0000000_;bl0080oooo00`0
^;Rh0000003Kfm/0oooo05ADE01dM7@07AdM0>KViP2l_;`08R8R06i^KP1oOgl40?ooo`030<;2`P0E
5AD0U9BD00X0oooo00<0i^KV01`L702PX:00U`3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=0?elo@80N7Qh3@000002
07QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1hN7P=07con`80N7Qh3@1loo/2
07QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00P0`l?30?3`l03oool0=SHf075a
L@3Zj^X08B4Q06a/K080oooo00<0_[jn0000003Vi^H00P3oool0303Ti>@000000<71`@3kno/03`l?
0;jn_P1YJFT0VIVI07EeM@1WIfL0`/;202/[:`80oooo00D0hN7Q04I6AP0410@0MWIf0?KfmP0;0?oo
o`030=WIf@0L71`0YJFU09H0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0k>c/00<0Ogmo0>c/k03/
k>`00`3/k>`00`1hN7P0oGcm0?elo@0;0?elo@80N7Qh3@3mO?d207QhN0d000000P1hN7P=0?_m>`80
N7Qh3@3mO?d207QhN0d000000P1hN7P=07con`80N7Qh3@1loo/207QhN0d0O?ok0P1hN7P=08;oO003
07QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0080<?3``3`l?00oooo03Hf=P071`L00@410000002j
^[X20?ooo`030=gMg@000000/[:b0080oooo0140/;2`0000003QhN40`<3000d=3@3jn_X0^;Rh04U9
B@0_;bl0[:b/0?gmo@0H61P0emOG0?ooo`0o?cl0AdM70>3Ph00>0?ooo`030=gMg@0N7Qh0XZ:R09D0
oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P0oGcm
0?elo@0;0?elo@80N7Qh3@3mO?d207QhN0d000000P1hN7P=0?_m>`80N7Qh3@3mO?d207QhN0d00000
0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0O?ok0P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`
0P1oOgl:0?3`l0090<?3``3`l?00oooo03Hf=P1gMgL0j^[Z0:>SX`061PH0[jn_0080oooo01L0ATI6
04=3@`3GemL0e]KF0492@P19BDT0oooo07MgM`1@D500oooo0?Win@0C4a<00`<30>k^kP3oool0JFUY
08>3P`3oool0>SXj092@T03no_h0gmoO0?Win@020?ooo`030<G5a@2j^[X0oooo00P0oooo00<0c/k>
00T92@32`/80U03oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN03mO?d0oGcm00/0oGcm0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?el
o@80N7Qh3@00000207QhN0d0O?ok0P1hN7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0?3`
l03`l?000P3`l?0207moO`X0l?3`00D0`l?30?3`l03oool0=SHf07MgM`020?ooo`0=04e=C@1>CTh0
oooo0?ooo`3/k>`0GEeM01LG5`0H61P0GemO0>k^kP3oool0DE5A0:BTY0020?ooo`0<06YZJP1@D500
oooo0?ooo`32`/80E5AD0?ooo`35aLD0ADE5014A4@0N7Qh0jNWY0P3oool00`23Ph<0K6a/0?ooo`09
0?ooo`030492@P1ZJVX0oooo09<0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0k>c/00<0Ogmo0>c/
k03/k>`00`3/k>`00`1hN7P0oGcm0?elo@0;0?elo@80N7Qh3@3mO?d207QhN0d000000P1hN7P=0?_m
>`80N7Qh3@3mO?d207QhN0d000000P1hN7P=07con`80N7Qh3@1loo/207QhN0d0O?ok0P1hN7P=08;o
O00307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0050<?3``3`l?00oooo03Hf=P1gMgL00P3oool0
0`0b<S80F5QH0?ooo`0R0?ooo`060?gmo@3oool0oooo0?ooo`0`<300OgmoU03oool00`3`l?00`l?3
0>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03mO?d0oGcm00/0oGcm0P1h
N7P=0?elo@80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0O?ok0P1h
N7P=07con`80N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00T0
`l?30?3`l03oool0=SHf02l_;`1IFET08B4Q00`<3034a<@08`3oool01P1IFET0DE5A09VIV@1PH600
1PH60>CTi9@0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?2F
07QhN0@0l?3`0P1oOgl:0?3`l0040<?3``3`l?00oooo0;2`/080VYZJ00<0^k^k0?Win@3oool08`3o
ool01@3Zj^X0[Jf]07=cL`2KVi/0k>c/09D0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0l?3`00<0
Ogmo0?3`l03`l?000`3`l?2F07QhN0@0l?3`0P1oOgl:0?3`l0030<?3``3`l?00oooo0<<0oooo00<0
l?3`0<?3``3`l?002@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0nodk0?_m>`0;
0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=
07con`80N7Qh3@00000207QhN0d0O?ok0P1hN7P=08;oO00307QhN03/k>`0k>c/0080k>c/0P1oOgl:
0>c/k0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0k>c/00<0Ogmo0>c/
k03/k>`00`3/k>`00`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081n
o@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80N7Qh3@00000207QhN0d0O?ok0P1hN7P=08;o
O00307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3
``3/k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P0nodk0?_m>`0;0?_m>`80
N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80
N7Qh3@00000207QhN0d0O?ok0P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l003
0<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0l?3`00<0Ogmo0?3`l03`l?00
0`3`l?000`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh
3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80N7Qh3@00000207QhN0d0O?ok0P1hN7P=08;oO00307Qh
N03`l?00l?3`0080l?3`0P1oOgl:0?3`l0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3`l?00
2@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3m
O?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80N7Qh3@00
000207QhN0d0O?ok0P1hN7P=08;oO00307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0030<?3``3`
l?00oooo00d0oooo00<0m?Cd0>SXj03oool0103oool00`3]kNd0kno_0?ooo`040?ooo`030?Cdm03X
j>P0oooo00<0oooo0P3bl_8=0?ooo`030>SXj03Bd]80oOgm0900oooo00<0l?3`0<?3``3/k>`02@3/
k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3mO?d2
07QhN0d0P7km0P1hN7P=081no@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80N7Qh3@000002
07QhN0d0O?ok0P1hN7P=08;oO00307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0050<?3``3`l?00
oooo03Hf=P1gMgL01@3oool07P10@400Ogmo0?ooo`3oool0inOW04Y:BP030`<08b<S01LG5`31`L40
oooo0>GUi@10@4000000018B4P0M7Ad0n?Sh0?ooo`3WinL0BTY:00<30`0S8b<05aLG0<71`@3oool0
9BDU010@400P8200C4a<0?KfmP80oooo00<0KFe]059BDP3oool01@3oool01P1FEUH04A4A03Lg=`0H
61P0I6AT0?;blP80oooo00<051@D0000000000000P0000000`1DE5@0oooo0?ooo`240?ooo`030?3`
l033`l<0k>c/00T0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0?_m>`3koC/02`3k
oC/207QhN0d0oGcm0P1hN7P=081no@80N7Qh3@20O_d207QhN0d0oGcm0P1hN7P=0?elo@80N7Qh3@1l
oo/207QhN0d000000P1hN7P=07con`80N7Qh3@22og`00`1hN7P0l?3`0?3`l0020?3`l080Ogmo2P3`
l?001@33`l<0l?3`0?ooo`0f=SH0MgMg00D0oooo01h0@41007moO`3oool0oooo04=3@`11@D40h>3P
0?ooo`3clo<0l_;b0?ooo`11@D40;2`/0<_;b`3oool0f=SH0?gmo@3oool0@d=30451@@3Ph>00oooo
0??cl`3bl_80oooo0<71`@3kno/0o_kn03Hf=P2BTY820?ooo`030=_Kf`3De=@0oooo00D0oooo00H0
iNGU0?gmo@3oool0i>CT00h>3P1cLg<20?ooo`0305EEE@0^;Rh0i^KV0080i^KV00<0kno_0?ooo`3o
ool0Q03oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307Qh
N03koC/0nodk00/0nodk0P1hN7P=0?elo@80N7Qh3@20O_d207QhN0d0P7km0P1hN7P=0?elo@80N7Qh
3@3mO?d207QhN0d0O?ok0P1hN7P=00000080N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0?3`l03`l?00
0P3`l?0207moO`X0l?3`00D0`l?30?3`l03oool0=SHf07MgM`050?ooo`060410@01oOgl0oooo0=gM
g@000000^KVi103oool00`3Lg=`0000009JFUP040?ooo`030=gMg@000000^KVi00L0oooo00<0`/;2
01DE5@2DU9@03P3oool01`1HF5P0@d=30?ooo`3oool0i^KV01`L702PX:00R@3oool00`3`l?00`l?3
0?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03koC/0nodk00/0nodk0P1h
N7P=0?elo@80N7Qh3@20O_d207QhN0d0P7km0P1hN7P=0?elo@80N7Qh3@3mO?d207QhN0d0O?ok0P1h
N7P=00000080N7Qh3@1loo/207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00L0
`l?30?3`l03oool0=SHf05EEE@33`l<0inOW00<0oooo00H0@41007moO`3oool0_[jn0000000j>SX3
03/k>`040<?3``2n_[h000000<k>cP@0oooo00<0_[jn0000000j>SX00`0k>c/01`33`l<0oooo0>7Q
h@16ATH010@407IfMP3fm_H03@3oool00`3clo<04Q8B08J6QP030?ooo`030=WIf@0L71`0YJFU08P0
oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0nodk
0?_m>`0;0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=081no@80N7Qh3@3mO?d207QhN0d0oGcm
0P1hN7P=07con`80N7Qh3@00000207QhN0d0O?ok0P1hN7P=08;oO00307QhN03/k>`0k>c/0080k>c/
0P1oOgl:0>c/k00G0<?3``3`l?00oooo03Hf=P0R8R80:2PX00410@1CDe<0l_;b0?ooo`10@400Ogmo
0?ooo`3MgMd000000:n_[`3EeMD0c<c<00<30`3Vi^H0gMgM0000002?Shl0103oool02`3MgMd00000
0:n_[`3EeMD0c<c<00<30`3Vi^H0oooo03lo?`17AdL0h>3P00d0oooo00D0WIfM05QHF00S8b<0LG5a
0?3`l0040?ooo`030=gMg@0N7Qh0XZ:R08L0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0l?3`00<0
Ogmo0?3`l03`l?000`3`l?000`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1h
N7P=081no@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80N7Qh3@00000207QhN0d0O?ok0P1h
N7P=08;oO00307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l00<0<?3``3`l?00oooo03Hf=P1gMgL0
oooo0=[JfP0>3Ph0IFEU0?ooo`10@400Ogmo0P3oool01@14A4@0IFEU0?7al@2FUYH0;Bd]0080oooo
00D0A4A402@T902e]KD0nOWi0=GEe@020?ooo`0504A4A01UIFD0lO7a09JFUP0];Bd00P3oool01@0j
>SX0T92@0?knoP3Ogml0nOWi0080oooo00<0aLG50;Zj^P3oool01P3oool0102][Jd0JVYZ03`l?02^
[Zh60?ooo`030<k>cP092@T0`/;208H0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0l?3`00<0Ogmo
0?3`l03`l?000`3`l?000`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3mO?d207QhN0d0P7km0P1hN7P=
081no@80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=07con`80N7Qh3@00000207QhN0d0O?ok0P1hN7P=
08;oO00307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l0050<?3``3`l?00oooo03Hf=P1gMgL00P3o
ool01@1ZJVX07QhN0?ooo`1MGEd0Ti>C0080oooo00D0j^[Z059BDP0A4A40:2PX0;jn_P020?ooo`05
0>c/k01RHV806a/K00h>3P0a<C400P3oool01@3Zj^X0DU9B014A4@0X:2P0_[jn0080oooo00D0aLG5
04E5A@0A4A407QhN0>WYj@020?ooo`0308>3P`1/K6`0oooo00P0oooo00<0h>3P00@4103De=@01P3o
ool00`12@T80JVYZ0?ooo`250?ooo`030?3`l033`l<0l?3`00T0l?3`000:0>c/k00307moO`3/k>`0
k>c/00<0k>c/00<0N7Qh0?_m>`3koC/02`3koC/207QhN0d0oGcm0P1hN7P=081no@80N7Qh3@20O_d2
07QhN0d0oGcm0P1hN7P=0?elo@80N7Qh3@1loo/207QhN0d000000P1hN7P=07con`80N7Qh3@22og`0
0`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`01@33`l<0l?3`0?ooo`0f=SH0MgMg0080oooo00<0
@d=304M7A`3oool0:`3oool00`3no_h06a/K09fMW@020?ooo`060?gmo@3oool0oooo0?ooo`0`<300
OgmoQP3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh
103/k>`207moO`X0k>c/00`0`l?30?3`l03oool0=SHf02l_;`1JFUX09RHV00820P2_[jl0oooo05]K
F`2ATI4V0?ooo`0609>CT`1^KVh0WinO05iNGP0;2`/0kno_0P3oool01P1IFET0DE5A09VIV@1PH600
1PH60>CTi8H0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?2F
07QhN0@0l?3`0P1oOgl:0?3`l0080<?3``3`l?00oooo0;2`/02JVYX0Vi^K0:f][@3Ng]h20?ooo`03
0;Bd]03=cLd0oooo02D0oooo00D0eMGE092@T01/K6`0VIVI0=gMg@030?ooo`050>[ZjP2][Jd0Lg=c
09^KV`3/k>`0Q`3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN03koC/0nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m
>`80N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`
l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03koC/0nodk00/0nodk0P1hN7P=0?doAP80
N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80
N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03o
ool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307Qh
N03koC/0nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh
3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/k>`0
0P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03koC/0nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00
000207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3k
oC/207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3o
ool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03koC/0
nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3koC/2
07QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?02
07moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1o
Ogl0k>c/0>c/k0030>c/k00307QhN03koC/0nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00000207Qh
N0d0nodk0P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3koC/207Qh
N0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00H0`l?30?3`l03oool0oOgm0=cLg03V
i^H60?ooo`030?Cdm03Xj>P0oooo00@0oooo00<0kNg]0>o_k`3oool0103oool00`3/k>`0kno_0?oo
o`0<0?ooo`030?_kn`3Xj>P0oooo00D0oooo0P3bl_8>0?ooo`030>[ZjP3EeMD0no_k01T0oooo00<0
m?Cd0=oOg`3oool03@3oool00`3dm?@0gmoO0?ooo`050?ooo`030>[ZjP3EeMD0no_k00D0oooo00<0
j^[Z0=GEe@3kno/0AP3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k003
0>c/k00307QhN03koC/0nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=
0?_m>`80N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3koC/207QhN0d0000000<0N7Qh
0>c/k03/k>`00P3/k>`207moO`X0k>c/00P0`l?30?3`l02`/;002PX:00d=3@0T92@0=SHf0:JVYP80
oooo01@0inOW04Y:BP030`<08b<S01LG5`31`L40oooo0>GUi@10@4000000018B4P0M7Ad0n?Sh0?oo
o`3Kfm/0=CDe00l?3`0<30`0>CTi0=cLg080oooo00H0@41007moO`3oool0oooo0:>SX`0L71`20?oo
o`060;Fe]@0>3Ph00P8202d];@2CTi<04A4A0P3oool01@0U9BD0410@020P801<C4`0m_Kf0080oooo
00<0KFe]059BDP3oool01@3oool01P3inOT0FUYJ00L71`0X:2P07alO0<c<c080oooo00H0mOGe00l?
3`0<30`0000000h>3P0F5QH20?ooo`0:04E5A@0Y:BT0oooo0?ooo`3emOD03`l?00`<300000003Ph>
01HF5P80oooo00H0h>3P04e=C@0F5QH0<S8b02l_;`2KVi/20?ooo`060?Gem@0?3`l030`<0000000>
3Ph05QHF0P3oool01P3Ph>00CDe=01HF5P0b<S80;bl_09^KV`80oooo00H0nOWi05YJFP071`L0:2PX
01lO7`3<c<`20?ooo`060?Win@1JFUX01`L702PX:00O7al0c<c<A@3oool00`3`l?00`l?30>c/k009
0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN03koC/0nodk00/0nodk0P1hN7P=0?do
AP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?do
AP80N7Qh3@3koC/207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`0280`l?30?3`
l03Fe]H0_kno0?[jnP3oool0WIfM00@4103Ng]h0oooo04=3@`11@D40h>3P0?ooo`3clo<0l_;b0?oo
o`11@D40;2`/0<_;b`3oool0f=SH0?gmo@3no_h0<c<c05]KF`3alO40kno_05IFEP0e=CD0o_kn0?oo
o`10@400Ogmo0P3oool02P2SXj<071`L0?ooo`3oool0<c<c05aLG03^k^h0b/[:02l_;`0A4A420?oo
o`050<71`@3kno/0o_kn03Hf=P2BTY800P3oool00`3Kfm/0e=CD0?ooo`050?ooo`0708N7Q`0X:2P0
hn?S0?ooo`2:RXX08b<S0?[jnP030?ooo`030=GEe@000000j>SX00<0oooo00<0UYJF08J6QP3oool0
0`3oool00`3EeMD000000>SXj0030?ooo`07051@D00f=SH0m_Kf0?ooo`31`L400@410=3@d0030?oo
o`030=GEe@000000j>SX00<0oooo01L0D51@03Hf=P3fm_H0oooo0<71`@010@40d=3@0?ooo`27QhL0
:2PX0>?Sh`3oool0RXZ:02<S8`3jn_X0oooo08N7Q`0X:2P0hn?S0?ooo`2:RXX08b<S0?[jnP140?oo
o`030?3`l033`l<0l?3`00T0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0?_m>`3k
oC/02`3koC/207QhN0d0oCm60P1hN7P=00000080N7Qh3@3koC/207QhN0d0nodk0P1hN7P=0?_m>`80
N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?_m>`80N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080
Ogmo2P3`l?000`33`l<0l?3`0?ooo`030?ooo`060>7Qh@000000]kNg0=gMg@000000^KVi103oool0
0`3Lg=`0000009JFUP040?ooo`030=SHf0000000_;bl0080oooo00H0^;Rh0000003Kfm/0oooo0410
@01oOgl20?ooo`060:>SX`0L71`0oooo0>OWi`010@40a/K60P3oool00`2/[:`04A4A0?ooo`030?oo
o`030<;2`P0E5AD0U9BD00X0oooo00L08B4Q07alO03oool0oooo0>[ZjP000000d=3@00<0oooo00<0
eMGE0000003Xj>P02@3oool00`3EeMD000000>SXj0030?ooo`0703De=@2;Rh/0oooo0?ooo`3gmoL0
20P80:^[Z`030?ooo`030=GEe@000000j>SX00<0oooo00X0=CDe08^;R`3oool0oooo0?Ogm`0820P0
Zj^[0?ooo`0Q8B40O7al0P3oool01P3Zj^X000000=3@d03oool08B4Q07alO080oooo00<0j^[Z0000
003@d=00A03oool00`3`l?00`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k003
07QhN03koC/0nodk00/0nodk0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`80
N7Qh3@3koC/207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3koC/207QhN0d0000000<0N7Qh0>c/k03/
k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool00P3oool01`3^k^h0C4a<00T92@3clo<0_[jn
0000000j>SX00`0k>c/01033`l<0_[jn0000003>c/h40?ooo`030;jn_P000000i^KV0080oooo00H0
i>CT00000031`L40oooo0410@01oOgl20?ooo`060:>SX`0L71`0oooo0<30`0000000i^KV0P3oool0
2@2/[:`04A4A0?ooo`3oool0hN7Q04I6AP0410@0MWIf0?KfmP090?ooo`030?Shn0010@40UiNG0080
oooo00<0e]KF0000003Lg=`00`3oool00`3EeMD000000>SXj0090?ooo`030=GEe@000000j>SX00<0
oooo00L0Ogmo04i>CP3oool0lo?c05YJFP0:2PX0lO7a00<0oooo00<0eMGE0000003Xj>P00`3oool0
2P1oOgl0CTi>0?ooo`3clo<0FUYJ00X:2P3alO40n?Sh00410@2GUiL20?ooo`060=KFeP000000g=cL
0?Shn0010@40UiNG0P3oool00`3Fe]H000000=cLg0140?ooo`030?3`l033`l<0k>c/00T0k>c/000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0?_m>`3koC/02`3koC/207QhN0d0oCm60P1hN7P=
00000080N7Qh3@3koC/207QhN0d0nodk0P1hN7P=0?_m>`80N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=
0?_m>`80N7Qh3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`04`33`l<0l?3`0?ooo`3o
ool0/;2`01TI6@0K6a/0a<C40?ooo`3MgMd000000:n_[`3EeMD0c<c<00<30`3Vi^H0gMgM0000002?
Shl0103oool00`3MgMd000000;:b/P020?ooo`060;2`/0000000hN7Q0?ooo`10@400FEUI0P3oool0
1P2LW9`07AdM0?ooo`3Rh^8000000;Fe]@80oooo00L0[:b/014A4@3oool0oooo03lo?`17AdL0h>3P
00/0oooo00L0i^KV0000000/;2`0dM7A0>g]k@1PH600?Cdm00@0oooo00<0eMGE0000003Xj>P02@3o
ool00`3EeMD000000>SXj0030?ooo`060?[jnP1KFe/0EEEE020P800R8R80bl_;103oool00`3EeMD0
00000>SXj0030?ooo`0F0?[jnP1KFe/0EEEE020P800R8R80bl_;0?ooo`3Vi^H0000002`/;03AdM40
kNg]061PH00m?Cd0oooo0>KViP000000;2`/0=7Ad@3]kNd0H61P03dm?DD0oooo00<0l?3`0<?3``3/
k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh
3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@3koC/207QhN0d0nodk0P1hN7P=0?doAP80N7Qh
3@3m?dH207QhN0d0nodk0P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l0070<?3
``3`l?00oooo07UiN@010@40GemO0>[ZjP030?ooo`0504A4A01UIFD0lO7a09JFUP0];Bd00P3oool0
1@14A4@092@T0;Fe]@3inOT0eMGE0080oooo00H0ATI604=3@`3GemL0e]KF0492@P19BDT20?ooo`06
0410@00H61P0KFe]0=CDe01BDU80DE5A0P3oool204Q8B0040>3Ph03Ti>@0C4a<014A4@80oooo00D0
>SXj092@T03no_h0gmoO0?Win@020?ooo`030<G5a@2j^[X0oooo00@0oooo00L0n_[j00820P2@T900
@D5101@D500j>SX0d=3@00@0oooo00<0eMGE0000003Xj>P02@3oool00`3EeMD000000>SXj0040?oo
o`0508j>SP020P80De=C04a<C03emOD0103oool00`3EeMD000000>SXj0040?ooo`0E08j>SP020P80
De=C04a<C03emOD0oooo0?[jnP020P80T92@0451@@0D51@0>SXj0=3@d03oool0n_[j00820P2@T900
@D5101@D500j>SX0d=3@04D0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0l?3`00<0Ogmo0?3`l03`
l?000`3`l?000`1hN7P0nodk0?_m>`0;0?_m>`80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80
N7Qh3@3koC/207QhN0d0nodk0P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0nodk0P1hN7P=00000003
07QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l0050<?3``3`l?00hn?S00410@2LW9`01@3oool01@3Z
j^X0DU9B014A4@0X:2P0_[jn0080oooo00D0k>c/069RHP0K6a/03Ph>034a<@020?ooo`060>c/k01M
GEd05aLG01PH601OGel0k^k^0P3oool01P1MGEd0TY:B08>3P`0N7Qh0>CTi0<W9b@80oooo00H0jn_[
04m?C`0B4Q80<C4a07AdM00A4A420?ooo`050<G5a@15ADD04A4A01hN7P3YjNT00P3oool00`23Ph<0
K6a/0?ooo`050?ooo`03024Q8@2FUYH0oooo00L0oooo00<0eMGE0000003Xj>P02@3oool00`3EeMD0
00000>SXj0030?ooo`060=CDe0010@40_Kfm0?ooo`2<S8`0FUYJ103oool00`3EeMD000000>SXj003
0?ooo`060=CDe0010@40_Kfm0?ooo`2<S8`0FUYJ0P3oool00`0Q8B40UYJF0?ooo`050?ooo`03024Q
8@2FUYH0oooo04P0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/
k>bF07QhN0@0k>c/0P1oOgl:0>c/k0050<?3``3`l?00fm_K00820P3WinL09`3oool00`2/[:`04A4A
0?ooo`0@0?ooo`0308B4Q00l?3`0o_kn00D0oooo00D0o_kn0?Win@3EeMD000000>SXj0070?ooo`05
0?knoP3inOT0eMGE0000003Xj>P00`3oool01P32`/808b<S0?ooo`3oool0bLW9014A4@80oooo00D0
o_kn0?Win@3EeMD000000>SXj0030?ooo`060<;2`P0S8b<0oooo0?ooo`39bLT04A4A0P3oool00`24
Q8@0?3`l0?knoP050?ooo`0308B4Q00l?3`0o_kn04P0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0
k>c/00<0Ogmo0>c/k03/k>`00`3/k>bF07QhN0@0k>c/0P1oOgl:0>c/k0080<?3``3`l?00oooo04];
B`0j>SX0SXj>065QH@1GEeLT0?ooo`030:b/[00A4A40oooo0100oooo00H0l?3`03Ti>@1ADE40XZ:R
07inOP1/K6`20?ooo`050?KfmP0C4a<0=3@d0000003Xj>P01`3oool01@3fm_H04a<C03@d=0000000
j>SX00@0oooo00D0<30`06][J`2JVYX0=cLg06UYJ@020?ooo`050?KfmP0C4a<0=3@d0000003Xj>P0
103oool01@0`<300Jf][09ZJVP0g=cL0JFUY0080oooo00H0l?3`03Ti>@1ADE40XZ:R07inOP1/K6`2
0?ooo`060?3`l00i>CT0DE5A0::RXP1nOWh0K6a/A@3oool00`3`l?00`l?30>c/k0090>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=0?doAP80N7Qh3@3m
?dH207QhN0d0oCm60P1hN7P=0?elo@80N7Qh3@3mO?d207QhN0d0oCm60P1hN7P=00000080N7Qh3@3k
oC/207QhN0d0nodk00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00P0`l?30?3`l03oool0no_k
0;Jf]P1^KVh0R8R80;no_b@0oooo00<0[:b/014A4@3oool04@3oool01@3hn?P0WYjN06a/K01oOgl0
^;Rh00<0oooo00@0nOWi0<C4a024Q8@0lO7a203oool0103inOT0a<C408B4Q03alO440?ooo`050?Ge
m@2[Zj/0LG5a0:n_[`3fm_H00`3oool0103inOT0a<C408B4Q03alO440?ooo`050?Gem@2[Zj/0LG5a
0:n_[`3fm_H00`3oool01@3hn?P0WYjN06a/K01oOgl0^;Rh00<0oooo00D0n?Sh09jNWP1/K6`0Ogmo
0;Rh^0150?ooo`030?3`l033`l<0l?3`00T0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0
N7Qh0000000000002`00000207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oGcm0P1h
N7P=0?elo@80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@3koC/00`1hN7P0l?3`0?3`
l0020?3`l080Ogmo2P3`l?000`33`l<0l?3`0?ooo`0Y0?ooo`030?knoP3jn_X0oooo09L0oooo00<0
l?3`0<?3``3`l?002@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;
00000080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=
0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`0307QhN03/k>`0k>c/0080k>c/0P1oOgl:
0>c/k0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0k>c/00<0Ogmo0>c/
k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?do
AP80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m
>`0307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3
``3/k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080
N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=0?doAP80
N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`0307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l003
0<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0l?3`00<0Ogmo0?3`l03`l?00
0`3`l?000`1hN7P000000000000;00000080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?doAP80N7Qh
3@3mO?d207QhN0d0oGcm0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`0307Qh
N03`l?00l?3`0080l?3`0P1oOgl:0?3`l0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3`l?00
2@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@3m
?dH207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=0?doAP80N7Qh3@00
000207QhN0d0nodk0P1hN7P=0?_m>`0307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0030<?3``3`
l?00oooo0<<0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`0
0`1hN7P000000000000;00000080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3mO?d2
07QhN0d0oGcm0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`0307QhN03/k>`0
k>c/0080k>c/0P1oOgl:0>c/k0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3/k>`02@3/k>`0
00X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@3m?dH207Qh
N0d0oCm60P1hN7P=0?doAP80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=0?doAP80N7Qh3@00000207Qh
N0d0nodk0P1hN7P=0?_m>`0307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l0030<?3``3`l?00oooo
0<<0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P0
00000000000;00000080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3mO?d207QhN0d0
oGcm0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk0P1hN7P=0?_m>`0307QhN03`l?00l?3`0080
l?3`0P1oOgl:0?3`l0030<?3``3`l?00oooo0<<0oooo00<0l?3`0<?3``3`l?002@3`l?0000X0k>c/
00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@3m?dH207QhN0d0oCm6
0P1hN7P=0?doAP80N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=0?doAP80N7Qh3@00000207QhN0d0nodk
0P1hN7P=0?_m>`0307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k0030<?3``3`l?00m?Cd00D0ADE5
00<0U9BD0?ooo`3dm?@01@15ADD00`2DU9@0oooo0?Cdm00504E5A@0309BDU03oool0m?Cd00D0ADE5
00<0U9BD0?ooo`3dm?@01@15ADD00`2DU9@0oooo0?Cdm00504E5A@0309BDU03oool0m?Cd00D0ADE5
00<0U9BD0?ooo`3dm?@01@15ADD00`2DU9@0oooo0?Cdm00504E5A@0309BDU03oool0m?Cd00D0ADE5
00<0U9BD0?ooo`3dm?@01@15ADD00`2DU9@0oooo0?Cdm00504E5A@0309BDU03oool0m?Cd00D0ADE5
00<0U9BD0?ooo`3oool0F`3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k00307QhN0000000000000/000000P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oCm60P1h
N7P=0?elo@80N7Qh3@3mO?d207QhN0d0oCm60P1hN7P=00000080N7Qh3@3koC/207QhN0d0nodk00<0
N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03lo?`01@3;bl/00`3QhN40oooo0?cl
o0050<_;b`030>7Qh@3oool0o?cl00D0bl_;00<0hN7Q0?ooo`3lo?`01@3;bl/00`3QhN40oooo0?cl
o0050<_;b`030>7Qh@3oool0o?cl00D0bl_;00<0hN7Q0?ooo`3lo?`01@3;bl/00`3QhN40oooo0?cl
o0050<_;b`030>7Qh@3oool0o?cl00D0bl_;00<0hN7Q0?ooo`3lo?`01@3;bl/00`3QhN40oooo0?cl
o0050<_;b`030>7Qh@3oool0o?cl00D0bl_;00<0hN7Q0?ooo`3lo?`01@3;bl/00`3QhN40oooo0?oo
o`1K0?ooo`030?3`l033`l<0k>c/00T0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh
0000000000002`00000207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d0oGcm0P1hN7P=
0?elo@80N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@3koC/00`1hN7P0l?3`0?3`l002
0?3`l080Ogmo2P3`l?000`33`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0l?3`00T0l?3`000:0?3`
l00307moO`3`l?00l?3`00<0l?3`UP1hN7P40?3`l080Ogmo2P3`l?000`33`l<0l?3`0?ooo`330?oo
o`030?3`l033`l<0l?3`00T0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/UP1hN7P40>c/k080
Ogmo2P3/k>`00`33`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0k>c/00T0k>c/000:0>c/k00307mo
O`3/k>`0k>c/00<0k>c/00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh
3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh
3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`00`33`l<0l?3`0?ooo`330?ooo`030?3`
l033`l<0k>c/00T0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh081no@20O_d02`20
O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m
?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0l?3`0?3`l0020?3`l080Ogmo2P3`
l?000`33`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0l?3`00T0l?3`000:0?3`l00307moO`3`l?00
l?3`00<0l?3`00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@3mO?d2
07QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@22og`0
0`1hN7P0l?3`0?3`l0020?3`l080Ogmo2P3`l?000`33`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0
l?3`00T0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh081no@20O_d02`20O_d207Qh
N0d0P7km0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207Qh
N0d000000P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`00`33
`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0k>c/00T0k>c/000:0>c/k00307moO`3/k>`0k>c/00<0
k>c/00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0
oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0
k>c/0>c/k0020>c/k080Ogmo2P3/k>`00`33`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0k>c/00T0
k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km
0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d00000
0P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0l?3`0?3`l0020?3`l080Ogmo2P3`l?000`33`l<0l?3`
0?ooo`330?ooo`030?3`l033`l<0l?3`00T0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0
N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0oGcm0P1h
N7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0l?3`0?3`
l0020?3`l080Ogmo2P3`l?000`33`l<0l?3`0?ooo`330?ooo`030?3`l033`l<0l?3`00T0l?3`000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=
00000080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1hN7P=
0?_m>`80N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`00`33`l<0l?3`0?ooo`03
0?ooo`030>CTi03@d=00m?Cd00@0oooo00<0k>c/0?gmo@3oool00P3dm?@?0?ooo`030?Cdm03Xj>P0
oooo00P0oooo00@0o_kn0=cLg03Kfm/0o_kn3@3oool00`3dm?@0j>SX0?ooo`260?ooo`030?3`l033
`l<0k>c/00T0k>c/000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh081no@20O_d02`20O_d2
07QhN0d0P7km0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH2
07QhN0d000000P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`0
5@33`l<0l?3`0?ooo`3oool0WYjN01@D500?3`l0<S8b01DE5@0P8200b/[:0?ooo`1ADE4020P801XJ
6P2LW9`04a<C05MGE`3oool0@41007moO`020?ooo`0607YjNP13@d<0oooo0?ooo`2d];@02@T90P3o
ool01P3WinL0BTY:00<30`0S8b<05aLG0<71`@H0oooo00L0Y:BT018B4P0P82008b<S00h>3P2BTY80
o_kn00<0oooo00<0OGem00410@3MgMd00`3oool02@3WinL0BTY:00<30`0S8b<05aLG0<71`@3oool0
@41007moO`220?ooo`030?3`l033`l<0k>c/00T0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`
00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@3mO?d207QhN0d0oGcm
0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1hN7P=0?_m>`80N7Qh3@22og`00`1hN7P0l?3`
0?3`l0020?3`l080Ogmo2P3`l?001P33`l<0l?3`0?ooo`2GUiL010@409FEU@80oooo00T0lO7a0000
002j^[X0a/K600@4103MgMd0i^KV05aLG00A4A400P3oool01P10@400Ogmo0?ooo`3oool0NWYj04=3
@`80oooo00X0];Bd00T92@3oool0oooo04=3@`11@D40h>3P0?ooo`3clo<0l_;b1@3oool00`2SXj<0
2PX:0;Ng]`020?ooo`030<O7a`0E5AD0QhN70080oooo00@0o_kn01lO7`0F5QH0NGUi0`3oool02@13
@d<0@D510>3Ph03oool0lo?c0?;blP3oool0@41007moO`220?ooo`030?3`l033`l<0l?3`00T0l?3`
000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1h
N7P=00000080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=08;oO080N7Qh3@3m?dH207QhN0d000000P1h
N7P=0?_m>`80N7Qh3@22og`00`1hN7P0l?3`0?3`l0020?3`l080Ogmo2P3`l?001@33`l<0l?3`0?Sh
n00C4a<0JVYZ00<0oooo00T0lO7a0000002j^[X0f][J00@4102k^k/0oooo09nOW`0P82000P3oool0
1P10@400Ogmo0?ooo`3oool0NWYj04=3@`80oooo00H0];Bd00T92@3oool0gMgM0000002i^KT80?oo
o`030?[jnP0J6QX0E5AD00@0oooo00d0Ti>C00`<303/k>`0oooo0;jn_P0;2`/0]KFe01PH603kno/0
oooo0=gMg@000000^KVi00D0oooo00<0@41007moO`3oool0P@3oool00`3`l?00`l?30?3`l0090?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN020O_d0P7km00/0P7km0P1hN7P=081no@80
N7Qh3@00000207QhN0d0oGcm0P1hN7P=0?elo@80N7Qh3@22og`207QhN0d0oCm60P1hN7P=00000080
N7Qh3@3koC/207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00D0`l?30?3`l03C
dm<000000<?3``030?ooo`060?7al@000000^[Zj0?ooo`2`/;00=SHf0P0c<c<01@0P8200oooo0?oo
o`10@400Ogmo0080oooo00X0NWYj04=3@`3oool0oooo0;Bd]0092@T0oooo0;jn_P000000>SXj0`0k
>c/00`33`l<0oooo0?ooo`020?ooo`030=GEe@000000VYZJ00@0oooo00d0hn?S0000002k^k/0oooo
05iNGP1OGel0oooo02PX:02][Jd0oooo0;jn_P000000>SXj00<0>c/k00@0`l?30?ooo`10@400Ogmo
PP3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN020
O_d0P7km00/0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0oGcm0P1hN7P=0?elo@80N7Qh3@22
og`207QhN0d0oCm60P1hN7P=00000080N7Qh3@3koC/207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/
k>`207moO`X0k>c/00D0`l?30?3`l02c/k<000000>?Sh`040?ooo`030?Ogm`3moOd0oooo00<0oooo
00H0UiNG024Q8@3oool0oooo0410@01ZJVX20?ooo`0>07QhN00a<C40oOgm0?ooo`2c/k<02@T90?oo
o`3MgMd000000:n_[`3EeMD0c<c<00<30`3Vi^H40?ooo`030;>c/`000000f=SH00@0oooo01@0no_k
00410@2HV9P0lo?c00/;2`2o_kl0oooo08f=S@18B4P0oooo0=gMg@000000[jn_0=GEe@3<c<`00`<3
0>KViP3oool0@41006IVIX80oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0l?3`00<0Ogmo0?3`l03`
l?000`3`l?2F07QhN0@0l?3`0P1oOgl:0?3`l0050<?3``3`l?00b<S80000003AdM401`3oool01@2]
[Jd0dM7A0?Shn01KFe/0EeMG0080oooo00X0@41001PH6023Ph<0gmoO03/k>`0T92@0FUYJ0=SHf01[
Jf/0=SHf0P3oool01@14A4@0IFEU0?7al@2FUYH0;Bd]00D0oooo00<0b<S80000002c/k<0103oool0
403?cll000000:f][@2NWYh08B4Q0?knoP3oool0jn_[00T92@3Ph>00oooo04A4A01UIFD0lO7a09JF
UP0];Bd20?ooo`050410@00E5AD0R8R80=[JfP3moOd0O`3oool00`3`l?00`l?30?3`l0090?3`l000
2P3`l?000`1oOgl0l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`00D0`l?30?3`l03^k^h0
0P8209ZJVP070?ooo`0509:BTP0m?Cd051@D04A4A039bLT00P3oool02P1MGEd0TY:B07]kN`0J6QX0
<c<c0=[JfP2SXj<09BDU02XZ:P2d];@20?ooo`050>[ZjP1BDU804A4A02PX:02n_[h01@3oool00`3_
knl00`<306e]K@040?ooo`0508R8R0000000emOG069RHP2=SHd00`3oool0201[Jf/0UiNG0?ooo`3Z
j^X0DU9B014A4@0X:2P0_[jn0P3oool01@1MGEd0T92@06QXJ00A4A40lO7a07l0oooo00<0l?3`0<?3
``3`l?002@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0P7km081no@0;081no@80
N7Qh3@20O_d207QhN0d000000P1hN7P=0?nN?`80N7Qh3@00000207QhN0d0P_ml0P1hN7P=0?elo@80
N7Qh3@00000207QhN0d000000P1hN7P=08;oO00307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/k006
0<?3``3`l?00oooo06EUI@0R8R80j^[Z903oool00`1ZJVX08B4Q0?7al@020?ooo`030?Shn00d=3@0
CTi>09@0oooo00<0l?3`0<?3``3/k>`02@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1h
N7P0P7km081no@0;081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0?nN?`80N7Qh3@00000207Qh
N0d0P_ml0P1hN7P=0?elo@80N7Qh3@00000207QhN0d000000P1hN7P=08;oO00307QhN03/k>`0k>c/
0080k>c/0P1oOgl:0>c/k00;0<?3``3`l?00oooo0>_[j`11@D407AdM07YjNP2GUiL0M7Ad02l_;`2n
_[h07`3oool01@3]kNd0@T9202LW9`23Ph<0RHV90080<S8b00<0h^;R0?ooo`3oool0TP3oool00`3`
l?00`l?30>c/k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN020O_d0P7km00/0
P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0
oGcm0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0
l?3`00/0`l?30?3`l03oool0oooo0?knoP2a/K40Ogmo06YZJP2;Rh/0[jn_0?7al@0P0?ooo`060?gm
o@2][Jd0NGUi07AdM02UYJD0nOWiU@3oool00`3`l?00`l?30?3`l0090?3`l0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l00307QhN020O_d0P7km00/0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0
oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=00000080N7Qh3@00000207QhN0d0
P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0`l?30?3`l03oool0``3oool00`3`l?00
`l?30?3`l0090?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN020O_d0P7km00/0P7km
0P1hN7P=081no@80N7Qh3@00000207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0oGcm
0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/
00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/k0090>c/k0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k00307QhN020O_d0P7km00/0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0oiho0P1h
N7P=00000080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml00<0
N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0`l?30?3`l03oool0``3oool00`3`l?00`l?30>c/
k0090>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN020O_d0P7km00/0P7km0P1hN7P=
081no@80N7Qh3@00000207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=
00000080N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0VYZJ
0;Vi^@33`l<0``33`l<00`2i^KT0VYZJ0?3`l0090?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN020O_d0P7km00/0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0oiho0P1hN7P=0000
0080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`
l03`l?000P3`l?0207moO`X0l?3`00<0JFUY07MgM`1lO7`0``1lO7`00`1gMgL0JFUY0?3`l0090?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN020O_d0P7km00/0P7km0P1hN7P=081no@80
N7Qh3@00000207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=00000080
N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moOm`0k>c/000:0>c/k00307mo
O`3/k>`0k>c/00<0k>c/00<0N7Qh081no@20O_d02`20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh
3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3mO?d207QhN0d000000P1hN7P=00000080N7Qh
3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`
l?000`1hN7P0P7km081no@0;081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0?nN?`80N7Qh3@00
000207QhN0d0P_ml0P1hN7P=0?elo@80N7Qh3@00000207QhN0d000000P1hN7P=08;oO00307QhN03`
l?00l?3`0080l?3`0P1oOgoL0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN020O_d0
P7km00/0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`2
07QhN0d0oGcm0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?02
07moOm`0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh081no@20O_d02`20O_d207Qh
N0d0P7km0P1hN7P=00000080N7Qh3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3mO?d207Qh
N0d000000P1hN7P=00000080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0
k>c/00<0Ogmo0>c/k03/k>`00`3/k>bF07QhN0@0k>c/0P1oOgl;0>c/k0030>[ZjP3;bl/0e=CD00H0
k>c/00L0dm?C0>[ZjP3/k>`0k>c/0=[JfP3Zj^X0k>c/0080hN7Q0`3/k>`00`3Cdm<0j^[Z0>c/k005
0>c/k080gmoO1P3/k>`00`3Cdm<0j^[Z0>c/k0060>c/k0030=_Kf`3MgMd0k>c/00<0k>c/0P3OgmnK
0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`00H0YJFU00T9
2@0<30`08B4Q038b<P2LW9`20?3`l00C0<K6aP0R8R8071`L0:n_[`3`l?00C4a<00L71`0H61P0TY:B
014A4@1ADE40l?3`0<K6aP0R8R8071`L0:n_[`3`l?00?3`l07MgM`020?3`l005028R8P0?3`l07QhN
04M7A`3WinL00P3`l?001`36a/H08R8R01`L702_[jl0l?3`03`l?01gMgL00P3`l?00303GemL0?3`l
0000000@41006a/K0>WYj@3`l?008R8R00l?3`0N7Qh0AdM70>OWi`80l?3`00<0IVIV04e=C@3`l?00
U03`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@00
000207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207QhN0d0P_ml0P1hN7P=0?elo@80N7Qh3@3m
O?d207QhN0d000000P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`0P1oOgl:0?3`l00B0<W9b@2c
/k<0jn_[0?3`l02CTi<00`<30=3@d03`l?00JVYZ03dm?@3_knl0l?3`0;Zj^P030`<0d=3@0=SHf01F
EUH0410@0P3`l?000`1ZJVX0?Cdm0>o_k`020?3`l00903`l?01gMgL0l?3`0?3`l02e]KD0k>c/0>o_
k`0b<S80RHV90080l?3`00<0JVYZ03dm?@3_knl00P3`l?00400l?3`0MgMg0?3`l03`l?00?Cdm02TY
:@2o_kl0l?3`0<_;b`3^k^h0l?3`0;Fe]@3/k>`0kno_038b<P29RHT20?3`l0030<k>cP37alL0l?3`
09@0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d0
00000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3mO?d207QhN0d0
oGcm0P1hN7P=00000080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo3P3/k>`01P3@d=00
00000:VYZ@3/k>`0AdM706UYJ@80k>c/00H0bLW900<30`2][Jd0k>c/09>CT`0M7Ad20>c/k00304M7
A`1YJFT0k>c/0080k>c/00<0>c/k07EeM@3/k>`00`3/k>`00`2c/k<04a<C08R8R0020>c/k00304M7
A`1YJFT0k>c/0080k>c/00H0>c/k07EeM@3/k>`0bl_;0000002:RXX70>c/k0030;>c/`0C4a<0R8R8
09T0k>c/000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d0
00000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3mO?d207QhN0d0
oGcm0P1hN7P=00000080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo3@3/k>`01`3Lg=`0
ATI600P8203Ph>00k>c/04M7A`1YJFT00`3/k>`01@2RXZ80<C4a02l_;`0_;bl07AdM0080k>c/00<0
AdM706UYJ@3/k>`00P3/k>`02@0k>c/0MGEe0>c/k03/k>`0d=3@0410@0030`<0KFe]0>?Sh`020>c/
k00304M7A`1YJFT0k>c/0080k>c/00H0>c/k07EeM@3/k>`0[jn_0000002n_[h50>c/k0050=3@d010
@4000`<306e]K@3Shn<0V@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000
000;00000080N7Qh3@00000207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207QhN0d0P_ml0P1h
N7P=0?elo@80N7Qh3@3mO?d207QhN0d000000P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`0P1o
Ogl<0?3`l0040:FUY@0G5aL06ATI0;Rh^080l?3`00<0B4Q806][J`3`l?001@3`l?001P2>SXh07alO
0?3`l03`l?00B4Q806][J`<0l?3`00L0?3`l07MgM`3`l?00l?3`03/k>`12@T80d];B00@0l?3`00<0
B4Q806][J`3`l?000P3`l?001P0l?3`0MgMg0?3`l03@d=00000008J6QPD0l?3`00<0>c/k0492@P3B
d]80V`3`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh
3@00000207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207QhN0d0P_ml0P1hN7P=0?elo@80N7Qh
3@3mO?d207QhN0d000000P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`0P1oOgl;0?3`l004075a
L@000000FEUI0=cLg080l?3`01@0iNGU0451@@1RHV80g=cL0>WYj@3`l?00XZ:R0<C4a03YjNT0EEEE
055AD@3`l?00iNGU0451@@1RHV80g=cL0>WYj@3`l?00?3`l07MgM`80l?3`00h0=SHf08N7Q`3_knl0
dM7A0>[ZjP3`l?00iNGU0451@@1RHV80g=cL0>WYj@3`l?00?3`l07MgM`80l?3`00D0@410024Q8@2Z
ZZX0j^[Z0<S8b0020?3`l00503Hf=P27QhL0kno_0=7Ad@3Zj^X00P3`l?000`2i^KT0[jn_0?3`l02D
0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=0000
0080N7Qh3@3oWSl207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=0?el
o@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moO`X0k>c/00<0d];B0000
002@T900103/k>`0501kNg/02PX:00l?3`0S8b<0Y:BT0>c/k027QhL0>3Ph018B4P0n?Sh0^[Zj0>c/
k01kNg/02PX:00l?3`0S8b<0Y:BT0>c/k01FEUH0R8R80P3/k>`03P2f][H0?clo00l?3`0K6a/0emOG
0>c/k01kNg/02PX:00l?3`0S8b<0Y:BT0>c/k01FEUH0R8R80P3/k>`01@3Jf]X0FUYJ01PH600<30`0
;Bd]0080k>c/00D0][Jf03lo?`0?3`l06a/K0=OGe`020>c/k00307UiN@1SHf<0k>c/09@0k>c/000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=
0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=
00000080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo2P3/k>`00`3:b/X00@410=GEe@05
0>c/k00304M7A`1YJFT0k>c/00T0k>c/00<0AdM706UYJ@3/k>`03@3/k>`00`17AdL0JFUY0>c/k02[
0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=0000
0080N7Qh3@3oWSl207QhN0d0oiho0P1hN7P=00000080N7Qh3@22og`207QhN0d0oGcm0P1hN7P=0?el
o@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moO`/0l?3`00D0ATI603Hf
=P25QHD0Fe]K055AD@020?3`l0030=;BdP3@d=00l?3`00T0l?3`00<0d];B0=3@d03`l?000P3`l?00
0`1EEED0R8R80?3`l0080?3`l0030=;BdP3@d=00l?3`0080l?3`00<0EEEE08R8R03`l?00YP3`l?00
00X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@00000207Qh
N0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207QhN0d0P_ml0P1hN7P=0?elo@80N7Qh3@3mO?d207Qh
N0d000000P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`0P1oOgl;0?3`l0050>c/k02[Zj/0IfMW
0820P02c/k<04`3`l?000`2YZJT0`<300?3`l00=0?3`l0030:VYZ@30`<00l?3`0:H0l?3`000:0>c/
k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=0?nN
?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3mO?d207QhN0d0oGcm0P1hN7P=0000
0080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0k>c/00<0Ogmo0>c/k03/
k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d0oiho0P1hN7P=0?nN?`80
N7Qh3@00000207QhN0d0P_ml0P1hN7P=0?elo@80N7Qh3@3mO?d207QhN0d000000P1hN7P=08;oO003
07QhN03/k>`0k>c/0080k>c/0P1oOgoL0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307Qh
N0000000000000/000000P1hN7P=00000080N7Qh3@3oWSl207QhN0d0oiho0P1hN7P=00000080N7Qh
3@22og`207QhN0d0oGcm0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`l?00
0P3`l?0207moOm`0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00
000207QhN0d000000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=08;oO080N7Qh3@3m
O?d207QhN0d0oGcm0P1hN7P=00000080N7Qh3@22og`00`1hN7P0l?3`0?3`l0020?3`l080Ogmog03`
l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>bF07QhN0@0k>c/0P1oOgoL0>c/k0002P3/k>`00`1o
Ogl0k>c/0>c/k0030>c/k9H0N7Qh103/k>`207moOm`0k>c/000:0?3`l00307moO`3`l?00l?3`00<0
l?3`00<0N7Qh0?nN?`3oWSl02`3oWSl207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207QhN0d0
00000P1hN7P=08;oO080N7Qh3@00000207QhN0d0oGcm0P1hN7P=00000080N7Qh3@22og`00`1hN7P0
l?3`0?3`l0020?3`l080Ogmog03`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P0oiho
0?nN?`0;0?nN?`80N7Qh3@3oWSl207QhN0d0oiho0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml
0P1hN7P=00000080N7Qh3@3mO?d207QhN0d000000P1hN7P=08;oO00307QhN03`l?00l?3`0080l?3`
0P1oOgoL0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03oWSl0oiho00/0oiho0P1h
N7P=0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=00000080N7Qh3@22og`207QhN0d000000P1h
N7P=0?elo@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moOm`0k>c/000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0?nN?`3oWSl02`3oWSl207QhN0d0oiho0P1hN7P=
0?nN?`80N7Qh3@00000207QhN0d000000P1hN7P=08;oO080N7Qh3@00000207QhN0d0oGcm0P1hN7P=
00000080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0l?3`00<0Ogmo0?3`
l03`l?000`3`l?000`1hN7P0oiho0?nN?`0;0?nN?`80N7Qh3@3oWSl207QhN0d0oiho0P1hN7P=0000
0080N7Qh3@00000207QhN0d0P_ml0P1hN7P=00000080N7Qh3@3mO?d207QhN0d000000P1hN7P=08;o
O00307QhN03`l?00l?3`0080l?3`0P1oOgoL0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l003
07QhN03oWSl0oiho00/0oiho0P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=00000080
N7Qh3@22og`207QhN0d000000P1hN7P=0?elo@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0?3`l03`
l?000P3`l?0207moOm`0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0?nN?`3oWSl0
2`3oWSl207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207QhN0d000000P1hN7P=08;oO080N7Qh
3@00000207QhN0d0oGcm0P1hN7P=00000080N7Qh3@22og`00`1hN7P0k>c/0>c/k0020>c/k080Ogmo
g03/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0oiho0?nN?`0;0?nN?`80N7Qh3@3o
WSl207QhN0d0oiho0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml0P1hN7P=00000080N7Qh3@3m
O?d207QhN0d000000P1hN7P=08;oO00307QhN03/k>`0k>c/0080k>c/0P1oOgoL0>c/k0002P3`l?00
0`1oOgl0l?3`0?3`l0030?3`l00307QhN03oWSl0oiho00/0oiho0P1hN7P=0?nN?`80N7Qh3@3oWSl2
07QhN0d000000P1hN7P=00000080N7Qh3@22og`207QhN0d000000P1hN7P=0?elo@80N7Qh3@000002
07QhN0d0P_ml00<0N7Qh0?3`l03`l?000P3`l?0207moOm`0l?3`000:0?3`l00307moO`3`l?00l?3`
00<0l?3`00<0N7Qh0?nN?`3oWSl02`3oWSl207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@00000207Qh
N0d000000P1hN7P=08;oO080N7Qh3@00000207QhN0d0oGcm0P1hN7P=00000080N7Qh3@22og`00`1h
N7P0l?3`0?3`l0020?3`l080Ogmog03`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0
oiho0?nN?`0;0?nN?`80N7Qh3@3oWSl207QhN0d0oiho0P1hN7P=00000080N7Qh3@00000207QhN0d0
P_ml0P1hN7P=00000080N7Qh3@3mO?d207QhN0d000000P1hN7P=08;oO00307QhN03/k>`0k>c/0080
k>c/0P1oOgoL0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN03oWSl0oiho00/0oiho
0P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d000000P1hN7P=00000080N7Qh3@22og`207QhN0d00000
0P1hN7P=0?elo@80N7Qh3@00000207QhN0d0P_ml00<0N7Qh0>c/k03/k>`00P3/k>`207moOm`0k>c/
000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0?nN?`3oWSl02`3oWSl207QhN0d0oiho0P1h
N7P=0?nN?`80N7Qh3@00000207QhN0d000000P1hN7P=08;oO080N7Qh3@00000207QhN0d0oGcm0P1h
N7P=00000080N7Qh3@22og`00`1hN7P0l?3`0?3`l0020?3`l080Ogmo5@3`l?001`3[k>h0iNW]0=[S
jP3GhNX0f^?Z0>GYk@3[k>h0`03`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?2F07QhN0@0l?3`
0P1oOglB0?3`l00=0>c]k`3UjNd0f^?Z0<gJi@31dn80^<gP0;?;g`2hcN00`M?R0<gJi@3JhnX0iNW]
0>c]k`2m0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/k>`207moOa40k>c/00l0
inSZ0=cRi`3:en80]/[L0:FoeP2H]m80Sk;@08bad02?/]00V;OB0:FoeP2fb]`0b]OR0=cRi`3Wj>X0
_03/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@3o
WSl207QhN0d0oiho0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@00
000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1oOgl@0>c/k00A0>OXjP3L
h^L0a=?P0:O0e@2@/Lh0PjS707bO_@1dTjh0L8fV07BC[P1lWkd0PjS7092acP2W`=D0a=?P0=cRi`3W
j>X0^`3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh
3@3oWSl207QhN0d0oiho0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh
3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl?0?3`l00C0>_/
kP3Pi^/0amKS0:W1e@2?[<D0O9^d07bO]P2<//D0XlgL0;SRl02PcM`0Sk;507jO]P1mVk@0Sjc50:W1
e@37e^<0h>K[0>_/kP2j0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN00000000000
00/000000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207Qh
N0d0P_ml0P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207mo
O`l0l?3`01<0iNW]0<gJiP2Y`]L0Sjc507^P]`27^LT0[^Wd0;C`n02on?`0`O[m0<;ho02el?P0/>Wd
08bib@1kX;L0Sjc50:W2e`3=f^H0iNW]0;X0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0
N7Qh0000000000002`00000207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@22og`207QhN0d0P_ml0P1h
N7P=08;oO080N7Qh3@22og`207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0k>c/0>c/
k0020>c/k080Ogmo3P3/k>`05@3XjN/0d]cU0;C9g02B//h0MYjm07^ge@2MgoX0[N_n0;C_o`2dkOl0
^?Co0;G^o`2glOl0[>[n0:GWnP20^mD0Mj2m09:bcP2dbM`0d]cU0>SYj`2i0>c/k0002P3/k>`00`1o
Ogl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=0?nN?`80N7Qh3@3oWSl207Qh
N0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=00000080N7Qh3@00000207Qh
N0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`h0k>c/01D0hNGY0<CDhP2P_MH0PJK506RNcP23
a?L0T<kn09[Ho`2RgOl0Xn3o0:SSo`2Th?l0X]go09[Go`2Edoh0PlCg06VNcP21Y/H0XKkG0<KFi03T
j>`0V`3_knl00`3^k^h0k>c/0>c/k00K0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307Qh
N0000000000000/000000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d0P_ml0P1hN7P=08;oO080N7Qh
3@22og`207QhN0d0P_ml0P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?00
0P3`l?0207moO`h0l?3`01D0f^?Z0;W>h@2C]]@0K9W306>Wj01b/oD0PL3n08_7o`2Dcol0U];o09oI
o`2JeOl0U<oo08[7o`23`Ol0LkGg05nSi01ST;X0Q:O509J[_P2V[kH0V@2`/;00102WYjL0YZJV0;Rh
^03Hf=PL0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1h
N7P=0?nN?`80N7Qh3@3oWSl207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1h
N7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`h0l?3`01D0
e=oY0;7:g`2</]80EXk3052Gg@1OXNH0L;7b07Rgm`22`Oh0RLOo08[7o`26aOl0PL3n07Ngm`1`/O80
GZ7V05:Gg01AR;d0OjG508jW_02LYk40V@2VYZH0102QXJ40UiNG08J6QP2TY:@L0?3`l0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=0?nN?`80N7Qh3@3oWSl2
07QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=00000080N7Qh3@000002
07QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`h0k>c/00T0cM[U0:c6g@29/=80BHO60429
dP1>U]h0GJ7U06FYk@1Z[O400P1b]?H02P1d]OL0K[3a06NYk@1IWn@0BI7L03j8d@17Pl@0Nj;408BN
]@2;V:>I09:BTP0408R8R026QXH0MGEe06][Ja`0k>c/000:0>c/k00307moO`3/k>`0k>c/00<0k>c/
00<0N7Qh0000000000002`00000207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@22og`207QhN0d0P_ml
0P1hN7P=08;oO080N7Qh3@22og`207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0k>c/
0>c/k0020>c/k080Ogmo3P3/k>`05@3=f^D0[LON08V`dP14OLD0=87;03J3dP16SmX0DYWQ05RNi01O
XnP0HjK[062Sj01GW^D0DYWQ04F>fP0]Om00;G_:04Ai``1fWKl0N9:Y07J3SP2I07IfMP04079bLP1[
Jf/0GemO08Z:RQ`0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00
000207QhN0d0oiho0P1hN7P=0?nN?`80N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22
og`207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmo3P3`
l?005@3@gNP0[lWP08bce@1;Lk`08Fk102=ebP1:SmH0IJ;O07F/h`1g[ND0MjkT07N^i@1[Y^40HIoM
04::e00WN<X08fo104I]]`1bVK/0JXBK05m/M`2I05IFEP0405ADE01EEED0QHF50<S8b1`0l?3`000:
0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00000207QhN0d0oiho0P1hN7P=
0?nN?`80N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d000000P1hN7P=
00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmo3P3`l?005@3DgnT0/lcQ096ge`1N
N[X09Enj01m_`P1IU]@0M:OM07n_h@24]>@0QKCT082ahP1i[>00JJ7K04nAd`0TK/40:5nj05ij^P2A
]mL0/lcQ0=COj@2i0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/0
00000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0
P_ml0P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`h0
k>c/00T0f>7X0;c@hP2K_MX0NIK1035C/P0kK<00KioG086_g@27]>400P2?^N802P2<]n40OjoN07>W
fP1LTm80=VVn039D/P1iU/40VkgJ0;c@hP3HhNRi0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/
k00307QhN0000000000000/000000P1hN7P=0?nN?`80N7Qh3@3oWSl207QhN0d0P_ml0P1hN7P=08;o
O080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/
k03/k>`00P3/k>`207moO`h0k>c/01D0h^KZ0<gJiP2ab]l0VKKB05Ub/P0jFkL0R:CG0:2mhP2S`N@0
Y<GW0:G6i`2O`ND0V[cR08f`g@1dV=40>UVe05Ub/P2I]]80/L[O0<gJiP3Ri^X0^@3/k>`000X0l?3`
00<0Ogmo0?3`l03`l?000`3`l?2F07QhN0@0l?3`0P1oOgl>0?3`l00E0>c]k`3Qin`0c=_X0;?;g`2:
Y/X0Dfba06=i`P2aa>@0_LkY0;g?jP2nd>X0_<gZ0;;6iP2Q]ml0GWO105=//@2:Y/X0/l_O0<cKj03Q
in`0k>g_0;T0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`UP1hN7P40?3`l080Ogmo3`3`l?00
4`3/kNl0hNO/0<cKj02cbml0RjS;061l]`1PN;h0R9gB0:Vkh02ibnH0Yk_P08FJd@1MMKd0H7bi08^X
b`2cbml0c=_X0>7Wk03/kNl0^P3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P00000
0000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0oCm6
0P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/
0P1oOgl@0>c/k00A0>SYj`3MhnP0bMSU0;7:g`2H^=H0OIo906R7_P1AKK<0A5j]055]/P1XQkd0OIo9
09RheP2ab]l0bMSU0=gSj03XjN/0^`3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0
00000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0
oCm60P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080
k>c/0P1oOglA0>c/k00?0>SYj`3Ni>T0cM[U0;S>h@2XaMh0V[gK09BjfP2A^=X0U;[J09Zmf`2XaMh0
^<kQ0<gJi@3Ni>T0j>W[0;`0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh00000000
00002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0?doAP80
N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080
Ogmo4P3`l?003@3/kNl0i^[^0=SRj`3:f^P0_M;U0;G>h`2bc><0]LkS0;gBi@3:f^P0f>;[0>KZkP3/
kNl0_@3`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh
3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0oCm60P1hN7P=0?doAP80N7Qh
3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOglD0?3`l0090>c]
k`3Vj^h0fnC[0=GPjP3Ag^T0eN3Z0=_Tj`3Vj^h0k>g_0;l0l?3`000:0>c/k00307moO`3/k>`0k>c/
00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1h
N7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P0
00000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0
oCm60P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080
k>c/0P1oOgoL0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/00000
0P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d0oCm6
0P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moOm`0l?3`
000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00000207QhN0d000000P1h
N7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0?doAP80N7Qh3@3m?dH207QhN0d000000P1h
N7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmog03`l?0000X0k>c/00<0Ogmo
0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=
0000000307QhN03/k>`0k>c/0080k>c/0P1oOgoL0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/
k00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/
k03/k>`00P3/k>`207moOm`0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh00000000
00002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0?doAP80
N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080
Ogmo4P3`l?000`2DU9@0/[:b0?3`l0370?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307Qh
N0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh
3@3m?dH207QhN0d0oCm60P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?00
0P3`l?0207moOa80l?3`00<0?3`l07MgM`3`l?00a`3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/
k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d0oCm60P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/
k>`0k>c/0080k>c/0P1oOgl;0>c/k0030>[ZjP3;bl/0e=CD00@0k>c/00D0>c/k07EeM@3[jn/0dm?C
0>_[j`050>c/k0030>7Qh@3Fe]H0k>c/00@0k>c/00<0hN7Q0=KFeP3/k>`00`3/k>`00`3Xj>P0e]KF
0>c/k02`0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/k>`207moO`X0k>c/00H0
XZ:R00T92@0<30`08B4Q034a<@2IVIT20>c/k00603/k>`15ADD08b<S01PH600C4a<0[Jf]0P3/k>`0
503EeMD0A4A400820P0P82005ADE0;:b/P3/k>`0eMGE04A4A0020P80820P01DE5@2b/[80k>c/0:NW
Y`0<30`00@4102TY:@28R8P03`l?0P3/k>`00`1TI6@0Bd];0>c/k02Y0>c/k0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO`X0l?3`02@0bLW90;>c/`3[jn/0l?3`09>CT`030`<0
d=3@0?3`l00l?3`0?clo0>3Ph03]kNd0OWin014A4@3Ti>@0l?3`03lo?`0m?Cd0d];B0?3`l03Ti>@0
hn?S0?3`l00o?cl0?Cdm0=;BdP3`l?00i>CT0>?Sh`3`l?00<30`05IFEP3Ph>00_[jn02`/;00@4102
0?3`l0030<k>cP37alL0l?3`0:T0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000
000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0?do
AP80N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`
l080Ogmo3P3`l?001P3Cdm<000000:b/[03`l?00?3`l07MgM`80l?3`00H0gMgM0000002QXJ40d=3@
0000002^[Zh40?3`l0030=3@d0000000[Zj^00@0l?3`00<0fMWI0000002j^[X00P3`l?000`2QXJ40
410@0?3`l02]0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/00000
0P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d00000
0P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO`d0k>c/
00L0g=cL04I6AP0820P0h>3P0>c/k00k>c/0MGEe00<0k>c/00D051@D08J6QP2_[jl0000003De=@03
03Hf=P040;Bd]02_[jl0000003De=@<0=SHf00@0];Bd0;6a/@000000e=CD0P3/k>`00`2OWil03`l?
0>c/k02]0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1h
N7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d000000P1h
N7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO``0k>c/00@0
XZ:R01LG5`0H61P0]KFe0P3/k>`0600k>c/0Kfm_0>c/k03/k>`0f][J0000002f][H0c<c<0000002Q
XJ40aLG50;bl_0020P80e=CD0<c<c0000000XJ6Q0<G5a@2l_;`00P820=CDe03AdM4000000:NWY`80
k>c/00<0WinO00l?3`3/k>`0[@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P00000
0000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0oCm6
0P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`
0P1oOgl;0?3`l004075aL@000000FEUI0=cLg0<0l?3`00d0?3`l01XJ6P25QHD0a/K605aLG00A4A40
kNg]0?3`l010@400GemO0>;RhP2=SHd0:RXZ0080l?3`00D0@41005mOG`3Rh^80SHf=02XZ:P020?3`
l080@d=300@0d];B0=KFeP17AdL0410@0P3`l?000`2i^KT0[jn_0?3`l02Y0?3`l0002P3`l?000`1o
Ogl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`X0l?3`00<0eMGE0000002BTY801@3`l?001P1G
EeL0R8R806UYJ@0E5AD08B4Q0:FUY@80l?3`00D0g=cL04e=C@0@41009BDU0;:b/P020?3`l0050=cL
g01=CDd0410@02DU9@2b/[800P3`l?001P3MgMd0BTY:010@400^;Rh0KFe]010@4080l?3`00<0Ng]k
06EUI@3`l?00Z@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;0000
0080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0oCm60P1hN7P=0000
0080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1oOgl:0>c/
k0030<[:bP010@40eMGE01l0k>c/00<0WinO00l?3`3/k>`0[@3/k>`000X0k>c/00<0Ogmo0>c/k03/
k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d0oCm60P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000003
07QhN03/k>`0k>c/0080k>c/0P1oOgl;0>c/k00504E5A@0e=CD0Ph>305UIF@1@D500703/k>`00`2O
Wil03`l?0>c/k02]0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO`/0
l?3`00D0k>c/0:^[Z`1WIfL0P8200;>c/`0L0?3`l0030:6QX@0@4100l?3`0:d0l?3`000:0?3`l003
07moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d000000P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=00000080
N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmo;03`l?000`3_knl0jn_[0?3`l02]0?3`l000
2P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moOm`0k>c/000:0>c/k00307moO`3/
k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d000000P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
00000`1hN7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?00
0`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d0oCm60P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00
l?3`0080l?3`0P1oOgoL0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN00000000000
00/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207mo
Om`0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/UP1hN7P40>c/k080Ogmog03/k>`000X0k>c/
00<0Ogmo0>c/k03/k>`00`3/k>bF07QhN0@0k>c/0P1oOgoL0>c/k0002P3`l?000`1oOgl0l?3`0?3`
l0030?3`l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1h
N7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0
N7Qh0?3`l03`l?000P3`l?0207moOm`0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh
0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=
0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l002
0?3`l080Ogmog03`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;0000
0080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0oCm60P1hN7P=0000
0080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1oOgoL0>c/
k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moOm`0k>c/000:0?3`l00307mo
O`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=0?doAP80N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh
3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmog03`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`
l?000`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d0oCm60P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03`
l?00l?3`0080l?3`0P1oOgoL0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000
000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH2
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`2
07moOm`0k>c/000:0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0?doAP80N7Qh3@00000207Qh
N0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmog03/k>`000X0
l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@00000207QhN0d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d0oCm60P1hN7P=00000080N7Qh3@00000207QhN0d0
00000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgoL0?3`l0002P3`l?000`1oOgl0l?3`
0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
0P1hN7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
00<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`D`3ooomm0?3`l0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1h
N7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0
N7Qh0>c/k03/k>`00P3/k>`207moO``0k>c/DP2<S8`00`3oool0k>c/0>c/k01k0>c/k0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO``0k>c/00<0S8b<0?ooo`3/k>`0CP3/k>`0
0`2<S8`0oooo0>c/k01l0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN00000000000
00/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@3m?dH207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207mo
O``0l?3`00<0S8b<0?ooo`3`l?00CP3`l?000`2<S8`0oooo0?3`l01l0?3`l0002P3`l?000`1oOgl0
l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO``0l?3`00@0S8b<0?ooo`3`l?00l?3`B`1hN7P20?3`
l00308b<S03oool0l?3`07`0l?3`000:0>c/k00307moO`3/k>`0k>c/00<0k>c/UP1hN7P40>c/k080
Ogmo303/k>`01@2<S8`0oooo0>c/k03/k>`0N7Qh00d000000P1hN7P=081no@80N7Qh3@20O_d207Qh
N0d000000P1hN7P=0000000507QhN03/k>`0k>c/08b<S03oool0O@3/k>`000X0k>c/00<0Ogmo0>c/
k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@00000207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=0000
000307QhN03/k>`0k>c/0080k>c/0P1oOgl<0>c/k00508b<S03oool0k>c/0>c/k01hN7P03@000002
07QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0000000D0N7Qh0>c/k03/k>`0S8b<0?oo
o`1m0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=
08;oO080N7Qh3@22og`207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`00D0S8b<
0?ooo`3`l?00l?3`07QhN00=00000080N7Qh3@20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@00
00001@1hN7P0l?3`0?3`l02<S8`0oooo07d0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0
N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1h
N7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@0000000`1hN7P0l?3`0?3`
l0020?3`l080Ogmo303`l?001@2<S8`0oooo0?3`l03`l?00N7Qh00d000000P1hN7P=081no@80N7Qh
3@20O_d207QhN0d000000P1hN7P=0000000507QhN03`l?00l?3`08b<S03oool0O@3`l?0000X0k>c/
00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d00000
0P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml
0P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1oOgl<0>c/k00508b<S03oool0k>c/0>c/k01h
N7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0000000D0N7Qh0>c/k03/
k>`0S8b<0?ooo`1m0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@22og`207QhN0d0
P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO``0
k>c/00D0S8b<0?ooo`3/k>`0k>c/07QhN00=00000080N7Qh3@20O_d207QhN0d0P7km0P1hN7P=0000
0080N7Qh3@0000001@1hN7P0k>c/0>c/k02<S8`0oooo07d0k>c/000:0?3`l00307moO`3`l?00l?3`
00<0l?3`00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@0000000`1h
N7P0l?3`0?3`l0020?3`l080Ogmo303`l?001@2<S8`0oooo0?3`l03`l?00N7Qh00d000000P1hN7P=
081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000000507QhN03`l?00l?3`08b<S03oool0O@3`
l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`2
07QhN0d0P_ml0P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl<0?3`l00508b<S03oool0
l?3`0?3`l01hN7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0000000D0
N7Qh0?3`l03`l?00S8b<0?ooo`1m0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN000
0000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@22
og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/
k>`207moO``0k>c/00D0S8b<0?ooo`3/k>`0k>c/07QhN00=00000080N7Qh3@20O_d207QhN0d0P7km
0P1hN7P=00000080N7Qh3@0000001@1hN7P0k>c/0>c/k02<S8`0oooo07d0k>c/000:0>c/k00307mo
O`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh
3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`01@2<S8`0oooo0>c/k03/k>`0N7Qh00d0
00000P1hN7P=081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000000507QhN03/k>`0k>c/08b<
S03oool0O@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml0P1hN7P=08;oO080
N7Qh3@22og`207QhN0d0P_ml0P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl<0?3`l005
08b<S03oool0l?3`0?3`l01hN7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207Qh
N0d0000000D0N7Qh0?3`l03`l?00S8b<0?ooo`1m0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=08;oO080N7Qh3@22og`207QhN0d0000000<0N7Qh0?3`
l03`l?000P3`l?0207moO``0l?3`00D0S8b<0?ooo`3`l?00l?3`07QhN00=00000080N7Qh3@20O_d2
07QhN0d0P7km0P1hN7P=00000080N7Qh3@0000001@1hN7P0l?3`0?3`l02<S8`0oooo07d0l?3`000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=
08;oO080N7Qh3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`01@2<S8`0oooo0>c/k03/
k>`0N7Qh00d000000P1hN7P=081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000000507QhN03/
k>`0k>c/08b<S03oool0O@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000
000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0P_ml0P1h
N7P=08;oO080N7Qh3@22og`207QhN0d0P_ml0P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1o
Ogl<0>c/k00408b<S03oool0k>c/0>c/k4/0N7Qh0P3/k>`00`2<S8`0oooo0>c/k01l0>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l9H0N7Qh103`l?0207moO``0l?3`00@0S8b<0?ooo`3`l?00l?3`
B`1hN7P20?3`l00308b<S03oool0l?3`07`0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`UP1h
N7P40?3`l080Ogmo303`l?001@2<S8`0oooo0?3`l03`l?00N7Qh00d000000P1hN7P=081no@80N7Qh
3@20O_d207QhN0d000000P1hN7P=0000000507QhN03`l?00l?3`08b<S03oool0O@3`l?0000X0k>c/
00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@00000207QhN0d00000
0P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
0P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1oOgl<0>c/k00508b<S03oool0k>c/0>c/k01h
N7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0000000D0N7Qh0>c/k03/
k>`0S8b<0?ooo`1m0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO``0
k>c/00D0S8b<0?ooo`3/k>`0k>c/07QhN00=00000080N7Qh3@20O_d207QhN0d0P7km0P1hN7P=0000
0080N7Qh3@0000001@1hN7P0k>c/0>c/k02<S8`0oooo07d0k>c/000:0?3`l00307moO`3`l?00l?3`
00<0l?3`00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1h
N7P0l?3`0?3`l0020?3`l080Ogmo303`l?001@2<S8`0oooo0?3`l03`l?00N7Qh00d000000P1hN7P=
081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000000507QhN03`l?00l?3`08b<S03oool0O@3`
l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl<0?3`l00508b<S03oool0
l?3`0?3`l01hN7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207QhN0d0000000D0
N7Qh0?3`l03`l?00S8b<0?ooo`1m0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN000
0000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/
k>`207moO``0k>c/00D0S8b<0?ooo`3/k>`0k>c/07QhN00=00000080N7Qh3@20O_d207QhN0d0P7km
0P1hN7P=00000080N7Qh3@0000001@1hN7P0k>c/0>c/k02<S8`0oooo07d0k>c/000:0>c/k00307mo
O`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh
3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`01@2<S8`0oooo0>c/k03/k>`0N7Qh00d0
00000P1hN7P=081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000000507QhN03/k>`0k>c/08b<
S03oool0O@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl<0?3`l005
08b<S03oool0l?3`0?3`l01hN7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh3@00000207Qh
N0d0000000D0N7Qh0?3`l03`l?00S8b<0?ooo`1m0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`
l03`l?000P3`l?0207moO``0l?3`00D0S8b<0?ooo`3`l?00l?3`07QhN00=00000080N7Qh3@20O_d2
07QhN0d0P7km0P1hN7P=00000080N7Qh3@0000001@1hN7P0l?3`0?3`l02<S8`0oooo07d0l?3`000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=
00000080N7Qh3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`01@2<S8`0oooo0>c/k03/
k>`0N7Qh00d000000P1hN7P=081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000000507QhN03/
k>`0k>c/08b<S03oool0O@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000
000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1h
N7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1o
Ogl<0>c/k00508b<S03oool0k>c/0>c/k01hN7P03@00000207QhN0d0P7km0P1hN7P=081no@80N7Qh
3@00000207QhN0d0000000D0N7Qh0>c/k03/k>`0S8b<0?ooo`1m0>c/k0002P3`l?000`1oOgl0l?3`
0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
0P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
00<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`00D0S8b<0?ooo`3`l?00l?3`07QhN00=00000080
N7Qh3@20O_d207QhN0d0P7km0P1hN7P=00000080N7Qh3@0000001@1hN7P0l?3`0?3`l02<S8`0oooo
07d0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00000207QhN0d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0
00000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmo303`l?001@2<S8`0
oooo0?3`l03`l?00N7Qh00d000000P1hN7P=081no@80N7Qh3@20O_d207QhN0d000000P1hN7P=0000
000507QhN03`l?00l?3`08b<S03oool0O@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1h
N7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/
0080k>c/0P1oOgl<0>c/k00408b<S03oool0k>c/0>c/k4/0N7Qh0P3/k>`00`2<S8`0oooo0>c/k01l
0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/k>`207moO``0k>c/00@0S8b<0?oo
o`3/k>`0k>c/B`1hN7P20>c/k00308b<S03oool0k>c/07`0k>c/000:0?3`l00307moO`3`l?00l?3`
00<0l?3`UP1hN7P40?3`l080Ogmo303`l?001@2<S8`0oooo0?3`l03`l?00N7Qh00d000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000507QhN03`l?00l?3`08b<S03oool0O@3`
l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl<0?3`l00508b<S03oool0
l?3`0?3`l01hN7P03@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000D0
N7Qh0?3`l03`l?00S8b<0?ooo`1m0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN000
0000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/
k>`207moO``0k>c/00D0S8b<0?ooo`3/k>`0k>c/07QhN00=00000080N7Qh3@00000207QhN0d00000
0P1hN7P=00000080N7Qh3@0000001@1hN7P0k>c/0>c/k02<S8`0oooo07d0k>c/000:0>c/k00307mo
O`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh
3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`01@2<S8`0oooo0>c/k03/k>`0N7Qh00d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000507QhN03/k>`0k>c/08b<
S03oool0O@3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P000000000000;00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080l?3`0P1oOgl<0?3`l005
08b<S03oool0l?3`0?3`l01hN7P03@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d0000000D0N7Qh0?3`l03`l?00S8b<0?ooo`1m0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`
l03`l?000P3`l?0207moO``0l?3`00D0S8b<0?ooo`3`l?00l?3`07QhN00=00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@0000001@1hN7P0l?3`0?3`l02<S8`0oooo07d0l?3`000:
0>c/k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=
00000080N7Qh3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`01@2<S8`0oooo0>c/k03/
k>`0N7Qh00d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000507QhN03/
k>`0k>c/08b<S03oool0O@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000
000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1h
N7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1o
Ogl<0>c/k00508b<S03oool0k>c/0>c/k01hN7P03@00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d0000000D0N7Qh0>c/k03/k>`0S8b<0?ooo`1m0>c/k0002P3`l?000`1oOgl0l?3`
0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
0P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
00<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`00D0S8b<0?ooo`3`l?00l?3`07QhN00=00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@0000001@1hN7P0l?3`0?3`l02<S8`0oooo
07d0l?3`000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00000207QhN0d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0
00000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmo303`l?001@2<S8`0
oooo0?3`l03`l?00N7Qh00d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
000507QhN03`l?00l?3`08b<S03oool0O@3`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1h
N7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/
0080k>c/0P1oOgl<0>c/k00508b<S03oool0k>c/0>c/k01hN7P03@00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d0000000D0N7Qh0>c/k03/k>`0S8b<0?ooo`1m0>c/k0002P3/k>`0
0`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO``0k>c/00D0S8b<0?ooo`3/k>`0k>c/07Qh
N00=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@0000001@1hN7P0k>c/0>c/
k02<S8`0oooo07d0k>c/000:0?3`l00307moO`3`l?00l?3`00<0l?3`00<0N7Qh0000000000002`00
000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d000000P1hN7P=00000080N7Qh3@0000000`1hN7P0l?3`0?3`l0020?3`l080Ogmo303`
l?001@2<S8`0oooo0?3`l03`l?00N7Qh00d000000P1hN7P=00000080N7Qh3@00000207QhN0d00000
0P1hN7P=0000000507QhN03`l?00l?3`08b<S03oool0O@3`l?0000X0l?3`00<0Ogmo0?3`l03`l?00
0`3`l?000`1hN7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307Qh
N03`l?00l?3`0080l?3`0P1oOgl<0?3`l00408b<S03oool0l?3`0?3`l4/0N7Qh0P3`l?000`2<S8`0
oooo0?3`l01l0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k9H0N7Qh103/k>`207moO``0k>c/
00<0S8b<0?ooo`3/k>`0CP3/k>`00`2<S8`0oooo0>c/k01l0>c/k0002P3/k>`00`1oOgl0k>c/0>c/
k0030>c/k9H0N7Qh103/k>`207moO``0k>c/00<0S8b<0?ooo`3/k>`0CP3/k>`00`2<S8`0oooo0>c/
k01l0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`00<0S8b<
0?ooo`3`l?00CP3`l?000`2<S8`0oooo0?3`l01l0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`
l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`
l03`l?000P3`l?0207moO``0l?3`00<0S8b<0?ooo`3`l?00CP3`l?000`2<S8`0oooo0?3`l01l0?3`
l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000000000/000000P1hN7P=00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`207moO``0k>c/00<0S8b<0?ooo`3/
k>`0CP3/k>`00`2<S8`0oooo0>c/k01l0>c/k0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307Qh
N0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh
3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`0
0P3/k>`207moO``0k>c/00<0S8b<0?ooo`3/k>`0CP3/k>`00`2<S8`0oooo0>c/k01l0>c/k0002P3`
l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=00000080N7Qh3@00
000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00
000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`00<0S8b<0?ooo`3`l?00CP3`
l?000`2<S8`0oooo0?3`l01l0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000
000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?02
07moO``0l?3`00<0S8b<0?ooo`3`l?004@3`l?000`3UiND0f][J0?3`l00<0?3`l0030=OGe`3^k^h0
l?3`0140l?3`00<0iNGU0=[JfP3`l?00103`l?000`3Ogml0h>3P0?3`l0040?3`l0030>GUi@3Jf]X0
l?3`00T0l?3`00<0S8b<0?ooo`3`l?00O03`l?0000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1h
N7P000000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207Qh
N0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/
0080k>c/0P1oOgl<0>c/k00308b<S03oool0k>c/00D0k>c/00<0<C4a09RHV03/k>`00P3/k>`00`2b
/[80000008n?S`020>c/k0090=GEe@14A4@00P82020P800E5AD0/[:b0<g=c@0C4a<0^[Zj0080k>c/
00P0NGUi01HF5P3EeMD0k>c/0<?3``0R8R806a/K0:b/[0D0k>c/00<0<C4a06i^KP3/k>`0103/k>`0
600k>c/0MGEe0>c/k03/k>`0eMGE04A4A0020P80820P01DE5@2b/[80k>c/0=?Cd`0k>c/00000010@
400J6QX0iNGU0>c/k03EeMD0A4A400820P0P82005ADE0;:b/PP0k>c/00<0S8b<0?ooo`3/k>`0O03/
k>`000X0k>c/00<0Ogmo0>c/k03/k>`00`3/k>`00`1hN7P000000000000;00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=0000000307QhN03/k>`0k>c/0080k>c/0P1oOgl<0>c/k00308b<S03oool0
k>c/00D0k>c/00P0<C4a09RHV03/k>`0k>c/0>?Sh`0Q8B40000008n?S`80k>c/00d0?Shn03`l?03?
cll0k>c/0>3Ph03Ogml0k>c/09fMW@0O7al0gmoO0:j^[P051@D0/[:b0080k>c/00<0J6QX03`l?03[
jn/01P3/k>`00`0a<C40KVi^0>c/k0040>c/k00H03/k>`1eMGD0k>c/0>c/k00n?Sh0?3`l0<o?c`3/
k>`0h>3P0=oOg`3/k>`0?3`l02PX:02k^k/0k>c/0<O7a`3Zj^X0k>c/03hn?P0l?3`0clo?0>c/k03P
h>00gmoO203/k>`00`2<S8`0oooo0>c/k01l0>c/k0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l003
07QhN0000000000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080
N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`
l?000P3`l?0207moO``0l?3`00<0S8b<0?ooo`3`l?001@3`l?00300b<S80Vi^K0?3`l03`l?00K6a/
014A4@0X:2P0TI6A0?3`l03@d=0000000:j^[PH0l?3`00@0HF5Q04M7A`0D51@0P8200`3`l?000`18
B4P0Jf][0?3`l0060?3`l003038b<P1`L700l?3`00@0l?3`00H0?3`l07MgM`3`l?00d=3@0000002^
[Zh40?3`l0030<o?c`000000SHf=00@0l?3`00<0d=3@0000002^[Zh0303`l?000`2<S8`0oooo0?3`
l01l0?3`l0002P3`l?000`1oOgl0l?3`0?3`l0030?3`l00307QhN0000000000000/000000P1hN7P=
00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=
00000080N7Qh3@00000207QhN0d0000000<0N7Qh0?3`l03`l?000P3`l?0207moO``0l?3`00<0S8b<
0?ooo`3`l?001@3`l?00300b<S80Vi^K0?3`l02l_;`00@4109ZJVP0k>c/0TI6A0?3`l02b/[800000
03Hf=P<0=cLg00L0]kNg0?3`l03`l?00hn?S00T92@0?3`l0inOW00<0l?3`00<0B4Q806][J`3`l?00
1P3`l?00100b<S80D51@0;Ng]`3IfMT30?3`l00603`l?01gMgL0l?3`0;:b/P000000=SHf0`0g=cL0
102g]kL0/[:b00000031`L440?3`l0030;:b/P000000=SHf00<0=cLg00<0]kNg0?3`l03`l?001P3`
l?000`2<S8`0oooo0?3`l01l0?3`l0002P3/k>`00`1oOgl0k>c/0>c/k0030>c/k00307QhN0000000
000000/000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@000002
07QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0000000<0N7Qh0>c/k03/k>`00P3/k>`2
07moO``0k>c/00<0S8b<0?ooo`3oool00`3oool20>c/k00@034a<@2HV9P0iNGU02HV9P15ADD0k>c/
03Xj>P2?Shl0k>c/0<c<c0000000XJ6Q0<G5a@2l_;`00P820=CDe080k>c/00@0J6QX024Q8@18B4P0
J6QX0`3/k>`00`17AdL0JFUY0>c/k0060>c/k00D034a<@0O7al09BDU0000001<C4`0gmoO0>c/k00k
>c/0MGEe0>c/k03<c<`000000:6QX@35aLD0_;bl00820P3De=@0c<c<00000024Q8@40>c/k0080<c<
c0000000XJ6Q0<G5a@2l_;`00P820=CDe03/k>`70?ooo`0308b<S03oool0k>c/07`0k>c/000:0>c/
k00307moO`3/k>`0k>c/00<0k>c/00<0N7Qh0000000000002`00000207QhN0d000000P1hN7P=0000
0080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000
0080N7Qh3@0000000`1hN7P0k>c/0>c/k0020>c/k080Ogmo303/k>`608b<S080k>c/00P0<C4a09RH
V01aLG4030`<0=;BdP3/k>`0>SXj08n?S`80k>c/00D0?Shn05eMG@3Ogml0RXZ:02TY:@020>c/k00<
09jNWP0820P0`<300=[JfP0H61P0/;2`0>c/k03QhN40@410065QH@3Hf=P0iNGU1@3/k>`02@0a<C40
KVi^0>c/k039bLT030`<05eMG@3/k>`0>c/k07EeM@020>c/k00503hn?P1MGEd0gmoO08Z:RP0Y:BT0
0P3/k>`01@0n?Sh08B4Q0:NWY`3Vi^H0aLG50080k>c/00D0?Shn05eMG@3Ogml0RXZ:02TY:@020>c/
k0P0S8b<00<0oooo0>c/k03/k>`0N`3/k>`000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?000`1hN7P0
00000000000;00000080N7Qh3@00000207QhN0d000000P1hN7P=00000080N7Qh3@00000207QhN0d0
00000P1hN7P=00000080N7Qh3@00000207QhN0d000000P1hN7P=0000000307QhN03`l?00l?3`0080
l?3`0P1oOglD0?3`l004038b<P1/K6`00`<308n?S`80l?3`00d0>c/k096AT@3`l?00l?3`0=cLg01=
CDd0410@02DU9@2b/[80l?3`0=_Kf`0`<300UIFE0080l?3`00P0Zj^[0451@@3[jn/0OGem00X:2P0@
41008b<S0:NWY`D0l?3`00T0<S8b071`L03`l?00l?3`06=SH`0L71`0l?3`05MGE`2:RXX00P3`l?00
1@3Lg=`0CDe=010@400U9BD0/[:b0080l?3`00D0g]kN05aLG00I6AT03@d=02h^;P020?3`l0050=cL
g01=CDd0410@02DU9@2b/[80R03`l?0000X0l?3`00<0Ogmo0?3`l03`l?000`3`l?2F07QhN0@0l?3`
0P1oOglD0?3`l004038b<P030`<0?3`l0>o_k`80l?3`00<0>c/k096AT@3`l?00403`l?000`18B4P0
Jf][0?3`l0060?3`l006038b<P1`L700l?3`0?3`l00o?cl0@T92X03`l?0000X0k>c/00<0Ogmo0>c/
k03/k>`0W@3/k>`207moOa@0k>c/00<0<C4a00P8203:b/X00`3/k>`00`0j>SX0Shn?0>c/k00@0>c/
k0030<o?c`3=cLd0k>c/00H0k>c/00T0<C4a02/[:`1CDe<08b<S00410@2QXJ40k>c/05ADE026QXH0
W@3/k>`000X0k>c/00<0Ogmo0>c/k03/k>`0W@3/k>`207moOa@0k>c/00<0XZ:R0:n_[`3/k>`00`3/
k>`00`2VYZH0alO70>c/k00I0>c/k0050::RXP2>SXh0Shn?0:2PX03=cLd00P3/k>`00`2VYZH0_Kfm
0>c/k02L0>c/k0002P3`l?000`1oOgl0l?3`0?3`l02M0?3`l080Ogmog03`l?0000X0l?3`00<0Ogmo
0?3`l03`l?00W@3`l?0207moOm`0l?3`000:0>c/k00307moO`3/k>`0k>c/09d0k>c/0P1oOgoL0>c/
k0002P3/k>bR07moOm`0k>c/003o0?3`l8T0l?3`003o0?3`l8T0l?3`003o0>c/k8T0k>c/003o0>c/
k8T0k>c/003o0?3`l8T0l?3`003o0?3`l8T0l?3`003o0>c/k8T0k>c/003o0>c/k8T0k>c/003o0?3`
l8T0l?3`003o0?3`l8T0l?3`003o0820P8T0P820003o0?Cdm8T0m?Cd003o0?OgmhT0moOg003o0?Og
mhT0moOg003o0?Cdm8T0m?Cd000H0?Cdm0030?3`l03EeMD0l?3`01/0m?Cd00<0j^[Z0=SHf03clo<0
3@3dm?@00`3`l?00eMGE0?3`l0040?Cdm0030>c/k03MgMd0m?Cd0?l0m?Cd>@3dm?@000H0moOg00<0
Bd];03lo?`3gmoL01@3gmoL01P1EEED0Bd];0?Ogm`3gmoL0EEEE04];B`80moOg00H0kno_06YZJP05
1@D03Ph>00X:2P1EEED<0?Ogm`0604];B`1bLW80moOg0?Ogm`2f][H0_kno0P3gmoL02P1WIfL0>c/k
0?Ogm`3gmoL0i^KV055AD@010@4030`<010@402GUiL40?Ogm`030:n_[`000000Shn?00<0moOg00H0
kno_06YZJP051@D03Ph>00X:2P1EEED20?Ogm`0503Ti>@0410@04a<C038b<P2k^k/0o`3gmoLh0?Og
m`001P3gmoL00`1;Bd/0?clo0?Ogm`050?Ogm`0605EEE@1;Bd/0moOg0?Ogm`1EEED0Bd];0P3gmoL0
1P1_Kfl04A4A0:FUY@3alO40kNg]0<c<c0`0moOg00X0Bd];079bLP3gmoL0i^KV00D51@0E5AD0lo?c
0?Ogm`1WIfL0>c/k0P3gmoL01`1KFe/09BDU0<71`@3^k^h0QhN70000002l_;`00`3gmoL00`1GEeL0
20P8038b<P030?Ogm`0606m_K`0A4A40YJFU0?7al@3]kNd0c<c<0P3gmoL01@2PX:00g]kN0?Ogm`1b
LW80=cLg0?l0moOg>03gmoL000H0m?Cd00<0BTY:03lo?`3dm?@01@3dm?@02P1DE5@0BTY:0?Cdm03d
m?@0E5AD04Y:BP3dm?@0k>c/00@4101YJFT@0?Cdm00>04Y:BP1`L700m?Cd09BDU0051@D07QhN0:j^
[P3dm?@0IVIV03Xj>P3dm?@0j>SX00410@1hN7P30?Cdm00=01DE5@1GEeL0m?Cd0?Cdm03YjNT02PX:
08b<S00410@0b/[:0?Cdm03/k>`010@406UYJ@090?Cdm00306e]K@0C4a<0l_;b0?l0m?Cd=`3dm?@0
00H0m?Cd00<0BTY:03lo?`3dm?@01@3dm?@02P1DE5@0BTY:0?Cdm03dm?@0E5AD04Y:BP3dm?@0b<S8
0000001eMGD309fMW@030:f][@3dm?@0m?Cd00X0m?Cd00h0BTY:071`L03dm?@0?clo04a<C01oOgl0
EUIF0?Cdm01VIVH0>SXj0?Cdm037alL000000:n_[`<0m?Cd00d0CDe=03@d=03dm?@0m?Cd09jNWP0;
2`/0jn_[04Y:BP1^KVh0m?Cd0<S8b0000000MGEe00<0WIfM00P0[Jf]0?Cdm03dm?@0m?Cd0<?3``0m
?Cd00@4108B4Q?l0m?Cd>03dm?@000H0moOg00<0Bd];03<c<`38b<P00P38b<P01@3Zj^X0moOg0?Og
m`1EEED0Bd];0080moOg00H0EEEE04];B`3gmoL0d=3@0000000c<c<204=3@`0301dM7@0g=cL0moOg
00/0moOg00h0Bd];079bLP3Ng]h00@410:NWY`3Jf]X030`<0>_[j`1WIfL0>c/k0?Ogm`3AdM400000
0:>SX`<0moOg00d0@D5103`l?03gmoL0moOg04Q8B01IFET0moOg0:NWY`0G5aL0m?Cd0=3@d0000000
<c<c0080@d=300P07AdM03Lg=`3gmoL0moOg0::RXP0410@0<c<c0;^k^ol0moOg>@3gmoL000H0moOg
00<0Bd];00<30`0>3Ph00P0>3Ph01@2f][H0moOg0?Ogm`1EEED0Bd];0080moOg00H0EEEE04];B`3g
moL0l?3`00/;2`2:RXX20?Ogm`0304I6AP1DE5@0moOg00/0moOg00h0Bd];079bLP2;Rh/03`l?0?3`
l03gmoL0@D5109bLW01WIfL0>c/k0?Ogm`3alO402PX:06YZJP80moOg00h0m?Cd00/;2`1WIfL0moOg
0>CTi00410@0/;2`0?Ogm`3alO404A4A0:n_[`3`l?002`/;08Z:RP80moOg00L0ATI605ADE03gmoL0
moOg03Ph>01IFET0m_Kf0?l0moOg>P3gmoL000H0m?Cd00<0BTY:03lo?`3dm?@01@3dm?@01P1DE5@0
BTY:0?Cdm03dm?@0E5AD04Y:BP80m?Cd00H0N7Qh02@T902b/[80WinO00/;2`2YZJT<0?Cdm00404Y:
BP1`L700<c<c065QH@80m?Cd00@0Vi^K0492@P1VIVH0>SXj0P3dm?@02`1kNg/0410@08>3P`2o_kl0
DE5A00L71`3EeMD0m?Cd092@T00D51@0l?3`0080m?Cd00T0IFEU055AD@3dm?@0N7Qh02@T902b/[80
WinO00/;2`2YZJT00P3dm?@01@1VIVH0A4A40<o?c`2k^k/0aLG50?l0m?Cd>03dm?@000H0m?Cd00<0
BTY:03lo?`3dm?@01@3dm?@01P1_Kfl0J6QX0?Cdm03dm?@0E5AD04Y:BP80m?Cd00D0l_;b07EeM@0I
6AT05aLG07moO`0=0?Cdm00404Y:BP1?Cdl000000;Vi^@80m?Cd00@0j^[Z00l?3`1>CTh0>SXj0P3d
m?@01P3clo<0PX:202@T900=3@d0>c/k0<?3``80m?Cd00<0H61P07QhN03dm?@00P3dm?@02032`/80
<30`0>_[j`3bl_80MGEe01TI6@0G5aL0Ogmo0`3dm?@01@3Fe]H0FUYJ01<C4`0F5QH0TI6A0?l0m?Cd
>03dm?@000H0moOg00<0Bd];03lo?`3gmoL01@3gmoL01P3GemL0eMGE0?Ogm`3gmoL0EEEE04];Ba@0
moOg00@0Bd];00P8200N7Qh0m_Kf0`3gmoL00`1MGEd01`L703/k>`3o0?OgmeL0moOg00060?Ogm`03
04];B`010@402PX:00802PX:00D051@D0?Ogm`3gmoL0EEEE04];B`020?Ogm`0305EEE@1;Bd/0moOg
01<0moOg00<0Bd];0000001gMgL0103gmoL00`2i^KT0000003/k>`3o0?OgmeL0moOg00060?Cdm003
0>3Ph03GemL0emOG0080emOG00D0fMWI0?Cdm03dm?@0hN7Q0>3Ph0020?Cdm00307EeM@1^KVh0m?Cd
01<0m?Cd00<0h>3P0=OGe`3[jn/0103dm?@00`3clo<0emOG0=kNgP3o0?Cdm5L0m?Cd003o0?Cdm8T0
m?Cd003o0?OgmhT0moOg003o0?OgmhT0moOg003o0?Cdm8T0m?Cd003o0?Cdm8T0m?Cd003o0?OgmhT0
moOg003o08b<S8T0S8b<003o0<[:bXT0b/[:000<0<c<c0060<g=c@3@d=00dm?C0=?Cd`3@d=00cLg=
3`3<c<`01P3=cLd0d=3@0=?Cd`3Cdm<0d=3@0<g=c@l0c<c<00H0cLg=0=3@d03Cdm<0dm?C0=3@d03=
cLgo0<c<c4d0c<c<00060<o?c`030<k>cP3?cll0clo?0080clo?00<0f=SH0>?Sh`3Xj>P00`3Ti>@0
0`3Shn<0f=SH0<o?c`020<o?c`030<k>cP3?cll0clo?00D0clo?00D0c/k>0<o?c`3Hf=P0hn?S0>SX
j0030>CTi0030>;RhP3Hf=P0clo?00D0clo?00<0c/k>0<o?c`3?cll0103?cll00`3Hf=P0h^;R0>SX
j0030>CTi0040>?Sh`3Hf=P0clo?0<k>cPL0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00D0clo?00<0c/k>0<o?c`3?cll01@3?cll00`3>c/h0clo?
0<o?c`050<o?c`030<k>cP3?cll0clo?00/0clo?00<0c/k>0<o?c`3?cll00@3?cll000X0dM7A00X0
h>3P0<O6aP2CQhH0QUiJ07m>AP20CTH0Rf5M09>6Q@38b<P0h>3P2`3AdM402P3Ph>00alK509>AQ028
QDl0QH4e08F2=@2>S500U9:40<S8b03Ph>0;0=7Ad@0:0>3Ph036a/D0S9>50766D@1VPCP0Ih<i07F<
D`2=U8D0b<S80>3Ph?l0dM7AB`3AdM4000T0dm?C00`0hN7Q09ZCT`26E4l0Zf1H0<Z8O`3OWiL0hZBJ
0<f:P`2_I5d0RUM@09VCT`3QhN490=?Cd`0<0>7Qh@2JUi80RGY20:nT@P3@bfL0i^B00>OSP@3BcfX0
]Ji508eoA02JUi80hN7Q2@3Cdm<0303QhN40UYRB06F0@`21[4P0Z/i[0<;RPP36i8H0[/mb08NbC01Z
R4D0U9RB0>7QhED0dm?C00<0`l?30=7Ad@3Cdm<00P39bLT30=?Cd`030<;2`P3<c<`0dm?C00X0dm?C
00<0bLW90;no_`3Cdm<0403Cdm<01@31`L40dm?C0=?Cd`32`/80dM7A0080dm?C00<0bl_;0<?3``3C
dm<01P3Cdm<00`3>c/h0_Kfm0=?Cd`030=?Cd`030<?3``3AdM40dm?C0080bLW93`3Cdm<00`3@d=00
^k^k0=3@d0040=?Cd`030<_;b`33`l<0dm?C00/0dm?C00<0_kno0=7Ad@3Cdm<0RP3Cdm<00080eMGE
1P3Fe]H03P3Lg=`0Vi6B08l^:@3HL6T0kiR@0?FYX03i]JX0nKJ/0?J]X`3`Vi<0ggM_09<a;02KTI80
g=cL1`3Fe]H03P3Lg=`0W9NA09Ye8`3K^e00m>Ag0?[eQ@3moI40oObB0?[hR03ejgT0h/ME09ik8`2L
Ui40g=cL1`3Fe]H03P3Lg=`0U9RA05F19P2EcED0_?1l0<gjS03KoID0fofF0=3jSP31lgd0WmYI05Z5
9P2DV940g=cLB03Fe]H02@0];Bd0RXZ:0=KFeP36a/H0QXJ60<C4a03Fe]H0X:2P00410@020=KFeP06
04=3@`061PH05ADE08:2PP0?3`l0BDU90P3Fe]H01P0o?cl030`<06=SH`3Fe]H0=CDe06YZJP80e]KF
00d0R8R801LG5`3Fe]H0e]KF0<71`@0n?Sh00P8201dM7@0C4a<0XJ6Q0=KFeP1mOGd08b<S0080e]KF
00<0[Jf]00000037alL00P3Fe]H05P071`L0UiNG0=KFeP2e]KD07alO00T92@0_;bl0Lg=c00H61P29
RHT0e]KF07emO@0=3@d092@T0=KFeP34a<@000000;:b/P3Fe]H0e=CD06ATI0051@D200d=3@0809BD
U03Fe]H0@d=300H61P0E5AD0PX:200l?3`19BDT70=KFeP030:NWY`000000_[jn00<0e]KF00H0eMGE
06e]K@071`L05QHF014A4@1TI6@20=KFeP0707emO@0=3@d092@T0=KFeP34a<@000000;:b/P030=KF
eP0503De=@1ZJVX0e]KF09bLW00@41000P0J6QX00`1bLW80e]KF0=KFeP1l0=KFeP80eMGE203Fe]H2
0=GEe@00203GemL03P2l^[X0PCPg0<MFD@3`PG`0n9F<0?ZQU`3jZ9l0nZ^Q0?ZTV@3hUXh0m8Am0<mJ
DP23=cH0^kZj1`3GemL03P2l^[T0U6P_0=nX?@3gdEl0oNMb0?odO@3onX@0oof50?oeOP3njg<0nMEQ
0>6]?@2EJC00_;Zh1`3GemL03P2i^[T0CW<^07ni?`2YgfH0_oMi0<_oP`3AohX0don;0<goQ031o7/0
ZnEX08Bm@@1@MBl0^KZhB03GemL0400];Bd0Rh^;0=OGe`1gMgL00000071`L03GemL0XJ6Q00410@3G
emL0YZJV00<30`2j^[X0`L7104e=C@0>3Ph20=OGe`030:JVYP010@40`l?30080emOG00H0=CDe06][
J`3GemL0emOG08V9R@0G5aL20=OGe`0403Ph>00f=SH0_;bl0=OGe`80c<c<00<0emOG07emO@0S8b<0
0P3GemL00`2^[Zh000000<S8b0020=OGe`0=00L71`2GUiL0emOG05iNGP0b<S80d];B0:RXZ00O7al0
EUIF0=OGe`3Fe]H061PH07emO@020=OGe`0@0<G5a@000000/k>c0=OGe`1nOWh00P8208Z:RP3Cdm<0
a/K60<C4a02VYZH00`<30;Zj^P31`L40CDe=00h>3PP0emOG00<0YjNW0000002o_kl00`3GemL02P20
P8002@T90::RXP3Fe]H0e=CD0<?3``3GemL0e]KF01PH601mOGd20=OGe`030<G5a@000000/k>c00<0
emOG00T0=CDe06][J`3GemL0_[jn0;Jf]P3GemL0^k^k00D51@2n_[h0R@3GemL000P0fMWI00h0RWQh
09hY903FH5T0kWQ_0?V5O03iThX0nYfC0?ZNU03iUH`0nHMn0>miK`3JHEX0XBPT08]hN0L0fMWI00h0
TX9h0<5k6`3a/D40nlUD0?gHH@3nhfl0onai0?o_NP3nig80o]YR0?_;E@3b]T40`g/K09>2N0L0fMWI
00h0NhAh056:6P2:`D@0X=]I0:oYJ02km7D0a?an0<GkO`2nnGT0/NeY0:7LG02<aTH0DX/J07b6N4P0
fMWI0100;Bd]08b<S03IfMT0:RXZ03dm?@0P8200fMWI0::RXP010@40fMWI0;Vi^@030`<0WinO0=WI
f@27QhL06a/K0P3IfMT00`29RHT05aLG0=WIf@020=WIf@0:03Hf=P1/K6`0fMWI0=WIf@2:RXX05aLG
0=WIf@2l_;`0000009fMW@D0fMWI00L0OWin02<S8`3IfMT0fMWI0;2`/0000000b/[:0080fMWI00d0
1`L709VIV@3IfMT0Lg=c01TI6@3>c/h0fMWI03hn?P1TI6@0fMWI0=;BdP000000ZJVY0080fMWI00H0
alO70000002e]KD0fMWI02TY:@0g=cL40=WIf@060;Vi^@030`<0WinO0=WIf@27QhL06a/K203IfMT0
0`2YZJT000000<71`@030=WIf@0302XZ:P1DE5@0fMWI00@0fMWI00<0d];B0000002YZJT00P3IfMT0
0`37alL000000;Fe]@030=WIf@0303Hf=P1/K6`0fMWI0080fMWI00@0e=CD07=cL`020P80`l?3R@3I
fMT000P0fm_K00H0KSdo0:Xa:P3>ED/0hVQM0?1fKP3eQ7d20?R=Q0060?J5OP3aN6h0iFUO0=9EB`2Z
<BT0L3do1`3Kfm/03P23ESd0c84W0?:Z=03l_D80oLYB0?gDH`3mfFX0oMY[0?gGI03mbe@0o;e20?B^
=P3@Q2L0QEHn1`3Kfm/03P15GSd0FY@U086m>P2CdDT0XMYK0:gUJ02ejfl0]N]`0:oXJ@2RgE/0Tm58
08;0>P1JUR@0AV<mB03Kfm/02@0^;Rh0SHf=0;Ng]`000000][Jf02@T902ZZZX0Y:BT00410@020=_K
f`0509NGU`0^;Rh0:b/[02/[:`0K6a/00P3Kfm/00`2;Rh/061PH0=_Kf`020=_Kf`0:03Hf=P1]KFd0
fm_K0=_Kf`2;Rh/061PH0=_Kf`2SXj<00000034a<@<0<S8b00@0YjNW0=_Kf`1oOgl092@T0P3Kfm/0
0`2a/K4000000<c<c0020=_Kf`0=00L71`2JVYX0fm_K0=SHf01YJFT08B4Q03Lg=`0D51@0IFEU0=_K
f`3De=@000000:^[Z`020=_Kf`060<S8b0000000][Jf0=_Kf`0@4100IfMW1@3Kfm/01@2GUiL0;Rh^
02/[:`0[:b/06a/K00P0fm_K00<0ZZZZ00000032`/800`3Kfm/00`0@4100820P038b<P02038b<P05
06m_K`3Kfm/0e=CD0000002[Zj/00P3Kfm/00`38b<P000000;Jf]P030=_Kf`0803Hf=P1]KFd0fm_K
0=_Kf`2KVi/061PH01HF5P2CTi>:0=_Kf`00203MgMd03P1I41@0YBX[0<A1>`3BDDP0g6=J0>E_J@3Z
N700jgUa0>MdJ`3PIE`0eU=90<Y2>`2U:BX0FQ0D1`3MgMd03P1j<1<0ag/[0>ZK8P3hZbX0o;Q20?c0
D03jaEL0o<MG0?c5C`3l_440nZ`[0>jN8@39OR/0Nc0C1`3MgMd03P0K?A80DXd[076`:01o_S80S/U6
09[DE@2Pf5/0X=QL09_EE`2?cDL0P<0a07>a:01BSb/07T4AB03MgMd02@0^;Rh0Shn?06YZJP0Y:BT0
gMgM07IfMP1JFUX0YJFU00410@050=gMg@0608:2PP0L71`0gMgM0=gMg@2<S8`06ATI0`3MgMd05P0g
=cL0CDe=0=gMg@3MgMd0QhN701TI6@3MgMd0_kno0000002GUiL0^;Rh0;2`/0020P80alO70=gMg@21
PH4061PH0=KFeP3MgMd0/[:b0000002l_;`20=gMg@0300H61P2LW9`0gMgM00<0gMgM00L0f][J03/k
>`1VIVH0gMgM0=KFeP000000[Jf]0080gMgM00H0b/[:0000002h^;P0gMgM02/[:`0b<S880=gMg@03
08:2PP0L71`0gMgM00L0gMgM00<0[:b/00000034a<@00`3MgMd02P0[:b/0FUYJ0;Rh^02h^;P0>3Ph
07inOP3MgMd0e]KF0000002][Jd20=gMg@030<[:bP000000XZ:R00<0gMgM00L0=cLg06i^KP3MgMd0
alO700820P1dM7@0eMGE08/0gMgM00080=oOg`0>04HB5@2C9R`0]2PU0<0h<039BT40dEU@0=EPEP3F
H5H0e5YA0<a;A033>Bl0]bTV09<V;01:4aH70=oOg`0>06PM5@2cIR/0fhLI0>VF4`3`XBH0lj/g0?Fa
?P3f/Sl0mJlh0?:T9@3/V1@0gX/J0;IW;01X7aH70=oOg`0>01/a500mNb/0F9`L06VZ6P1f]2/0Pkln
08S3@@29`d80Q<4m07Vh;01[[1X0G9lL03ek;00L<QA80=oOg`0902l_;`2?Shl06a/K07YjNP3Ogml0
b<S8014A4@2PX:000@410080gmoO0180UiNG0;Jf]P3Hf=P0Cdm?04a<C03Ogml0fm_K0861P@0G5aL0
c<c<0=;BdP3Ogml0=cLg01@D501OGel0^KVi04M7A`16ATH20=oOg`0503/k>`1HF5P0d];B08>3P`0W
9bL00P3Ogml0:022PX8030`<04E5A@2j^[X0LW9b00L71`0g=cL0YjNW0::RXP000000`l?30=oOg`3E
eMD0Rh^;0<[:bP3;bl/04a<C09JFUP3Ogml0b<S80000002PX:00c<c<0=oOg`3<c<`000000;Zj^P3O
gml0QHF500410@1kNg/0dm?C0<C4a03@d=00gmoO09NGU`2f][H0f=SH04m?C`1<C4`80=oOg`030:j^
[P000000a/K600<0gmoO01X0QHF5024Q8@30`<00^k^k010@402l_;`0gmoO0<S8b0000000X:2P0<c<
c03Ogml0c<c<0000000Y:BT0WIfM0<[:bP3Ogml0=cLg06m_K`3Ogml0a<C400H61P33`l<0fMWI0<;2
`XX0gmoO00080>3Ph00>04ln?`1e9BX0X2@T0;4V8`2m=c00`DDl0<=:@`34BT<0`THl0;df<02c9B<0
Wb<T07DV:P1D?T070>3Ph00>06<n?`2;@BD0aG0W0=R55`3PTQL0iI/R0>RN:03WX2P0iY`R0>>C5`3H
QaL0aG<W09Y8:P1S?Sh70>3Ph00>0415?`0/GRT0@XHU05FK6@1WYAX0LjdX07Fa;@1d/2`0LjhX06VX
7@1FWAT0@HPU02aN:@10ACm80>3Ph00402l_;`1KFe/00@410<W9b@80h>3P00<0?Cdm05eMG@010@40
0P3Ph>004P20P800=CDe014A4@0k>c/0/;2`0>3Ph02d];@05ADE00<30`0Q8B40GEeM0>3Ph00h>3P0
Kfm_07=cL`0J6QX0<S8b0;2`/080h>3P00D0cLg=04Q8B00>3Ph08b<S0:JVYP020>3Ph00:092@T011
@D40XJ6Q02XZ:P0G5aL0Rh^;0<30`00f=SH04a<C06QXJ080h>3P01`0emOG05IFEP0V9RH05ADE059B
DP3Hf=P0h>3P03@d=00000006QXJ024Q8@3Lg=`0dM7A024Q8@30`<00h>3P0=oOg`22PX808b<S00T9
2@0K6a/0Zj^[0>3Ph020P800=CDe014A4@0k>c/0/;2`203Ph>000`2^[Zh000000<O7a`030>3Ph005
0=oOg`1dM7@06ATI01<C4`1^KVh00P3Ph>003@0d=3@0000001XJ6P0Q8B40g=cL0=7Ad@0Q8B40Y:BT
03De=@11@D40h>3P055AD@21PH400P3Ph>00101mOGd09bLW00h>3P1;Bd^:0>3Ph0001P3Rh^804@3Q
hN40h^;R07mnOP1<71l0RCDj0:Tl?P2mDdl0`f=M0<ESGP35Hed0`V9L0;iBC`2Y?3h0RCHj04hM8020
Ogl0hN7Q00H0h^;R00H0QWin064P6`2]FcP0bX4o0=bKAP3QYT`20>6WCP060>2UB`3JVTH0bX8o0:aK
>@1c9B00QWmo0`3Rh^804P3QhN40h^;R0>;RhP3Rh^80OWin028d800nLCP0F9@n07N]AP23/dh0QKEB
08JfDP22/dh0MZa505VD?`0mL3P08cHP07moO`H0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40
h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40
h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40
h^;R0>;RhP050>;RhP050>7Qh@3Rh^80;bl_00l?3`0k>c/00`3Rh^800`2@T9003`l?00410@050>;R
hP060>7Qh@3Rh^80h^;R0>;RhP2?Shl06ATI0P3Rh^800`3QhN40>3Ph071`L0050>;RhP030>7Qh@3R
h^80h^;R00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40h^;R0>;RhP050>;RhP060>7Qh@3R
h^80h^;R0=[JfP000000/K6a0P3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R00D0
h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`2_[jl000000<W9b@050>;RhP030>7Qh@3Rh^80h^;R0080
h^;R00@0f][J0000002a/K40hN7Q1`3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R
00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R
00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R
00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R
00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R
00D0h^;R00<0hN7Q0>;RhP3Rh^801@3Rh^800`3QhN40h^;R0>;RhP050>;RhP030>7Qh@3Rh^80h^;R
00/0h^;R00<0hN7Q0>;RhP3Rh^800@3Rh^8000030=CDe03Shn<0i>CT00D0hn?S00h0][Jf03`a<P1L
=SL0WfY]0<VHV@3L/;80gkBe0=jd]@3L/;80bYNI09mZK@1N=cP0?S8c0;Ng]`L0hn?S00h0][Jf04d[
;`1l?CL0^7i/0=VbW03Xc[D0j]:h0>[B]`3Xc[D0fK:L0;UnK@21?cT0EB``0;Jf]PL0hn?S00H0][Jf
030`<00jBSX0KHa]09fmW02le;<20<?H^0060;gD/`2N_i`0K8a]03Y9>@0`<300][JfB03Shn<00`0`
<300000008b<S0030>?Sh`030=[JfP0;2`/00@4100T0hn?S00<0e]KF0;Jf]P3Shn<00P3Shn<00`0h
>3P0LG5a0>?Sh`0O0>?Sh`030>;RhP2g]kL0e]KF0080hn?S00<0d];B020P8032`/804P3Shn<01P2G
UiL0FEUI05UIF@16ATH00000051@D080FEUI00<0YZJV0>?Sh`3Shn<01@3Shn<00`3Rh^80]kNg0=KF
eP080>?Sh`03055AD@21PH40hn?S08`0hn?S0@3Ti>@10>?Sh`40e=CD00000`3EeMD0inOW0>KViP06
0>CTi00<08R9R@0R6a/0J5EF0:jKW03Dbl`0iMkN0>KPh03FcLd0[I^L06UEEP0S6a/0RHZ:2@3Ti>@0
1@2;R8P0@ahO08=CE`2nWI`0f/k<0080imkN00D0f/g<0;jMW022DeL0A1hR08b9R@090>CTi00<08R8
R00Q7R40EeYG09bSW03;cl`0gN7P0=kQgP3;d<d0Vj>L05QJF00P7B40RHV9B@3Ti>@00`2MWId0RHV9
0=SHf0040>CTi0030::RXP2:RXX0i>CT00d0i>CT00<0>CTi075aL@3Ti>@0903Ti>@00`3Lg=`0RHV9
0=GEe@0B0>CTi0030;:b/P29RHT0RHV900D0RHV900<0^k^k0>CTi03Ti>@0403Ti>@00`2PX:00]kNg
0>CTi02<0>CTi040i^KV0@3WinL10=_Kf`0000@0bl_;0>_[j`3Xj>P0inOW1P3Vi^H02P2;Rh/0=cHf
04e5AP1oMWH0VI2@09R@T01oMWH0CDE603Lf=P2;Rh/;0>KViP0:08b:RP18<c<0HT9408ieMP2ST900
Y92@08meMP1TADH0Ac<c08b:RP/0i^KV00X0RXZ:03@d=016ATH0MWIf092AT02ATY00MWIf04M7A`0d
=3@0RXZ:H@3Vi^H00`3Rh^80i>CT0>KViP3R0>KViP030>OWi`3Xj>P0jn_[0040e]KF0000102j^[X0
inOW0>[ZjP3Xj>P70>KViP030;Vi^@1dM7@0>c/k00804a<C00<0>c/k07AdM02i^KT03@3Vi^H00`2h
^;P0MWAd04Dj>`0202DA4`0304@j>P1fM7@0^;Rh00d0i^KV00<0^;Rh07AdM00k>c/00P0C4a<00`0k
>c/0M7Ad0;Rh^03o0>KViTP0i^KV00<0j>SX0>[ZjP3UiND00@3;bl/000060:VYZ@3=cLd0kNg]0>_[
j`3YjNT0inOWo`3Vi^Im0>KViP050>OWi`3YjNT0jn_[0>g]k@31`L400@33`l<000080;fm_@2b/[80
e]KF0?7al@3_knl0kNg]0>_[j`3YjNWo0>SXj7T0j>SX00L0jNWY0>_[j`3]kNd0kno_0?7al@3Fe]H0
WYjN0040gMgM00001`32`/80/[:b0:RXZ03<c<`0l?3`0??cl`3alO40o`3_knmk0>o_k`060?7al@3c
lo<0lO7a0=KFeP2c/k<0YZJV0@3alO40000808n?S`2CTi<0QXJ607IfMP2BTY80^[Zj0=SHf03Xj>So
0?WinGT0nOWi00L0j>SX0=kNgP3Fe]H0`L710;>c/`32`/80^;Rh0040m?Cd0000\
\>"], "Graphics",
  CellLabel->"From In[11]:=",
  Evaluatable->False,
  ImageSize->{392, 377},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"Tetris"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TetrisPlot", "Subsection",
  CellTags->"Tetris"],

Cell["Plotting the current Tetris board", "MathCaption",
  CellTags->"Tetris"],

Cell[CellGroupData[{

Cell[BoxData[
    \(TetrisPlot[$TetrisBoard]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"Tetris"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 2 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% DensityGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0192308 0.0961538 0.0384615 0.0961538 [
[ 0 0 0 0 ]
[ 1 2 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 2 L
0 2 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.01923 .03846 translate
.96154 1.92308 scale
30 string
10 20 8 [10 0 0 20 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000FF9E3FFF9E3F7CFFFB807EFD807EFDFD3F46000000FF9E3FFF9E3F
000000000000000000000000807EFD807EFDFD3F46000000FF9E3F000000
0000000000000000000000000000000000007CFFFB000000000000000000
0000000000000000000000007CFFFB7CFFFB7CFFFB000000000000000000
0000000000000000000000007CFFFB000000000000000000000000000000
0000000000000000000000007CFFFB000000000000000000000000000000
0000000000000000000000007CFFFB7CFFFB000000000000000000000000
000000000000000000000000000000FD3F46FD3F46000000000000000000
000000000000000000000000000000FD3F46000000000000000000000000
000000000000000000000000000000FD3F46000000000000000000000000
000000000000000000000000000000FD7CFD000000000000000000000000
000000000000000000000000000000FD7CFDFD7CFD000000000000000000
000000000000000000000000000000000000FD7CFD000000000000000000
0000000000000000000000000000000000007CFFFB000000000000000000
0000000000000000000000007CFFFB7CFFFB7CFFFB000000000000000000
000000000000000000000000807EFD807EFD000000000000000000000000
000000000000000000000000807EFD807EFD000000000000000000000000
000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000
pop
P
% End of image
.4 .4 .4 r
.25 Mabswid
[ ] 0 setdash
.01923 .03846 m
.01923 1.96154 L
s
.11538 .03846 m
.11538 1.96154 L
s
.21154 .03846 m
.21154 1.96154 L
s
.30769 .03846 m
.30769 1.96154 L
s
.40385 .03846 m
.40385 1.96154 L
s
.5 .03846 m
.5 1.96154 L
s
.59615 .03846 m
.59615 1.96154 L
s
.69231 .03846 m
.69231 1.96154 L
s
.78846 .03846 m
.78846 1.96154 L
s
.88462 .03846 m
.88462 1.96154 L
s
.98077 .03846 m
.98077 1.96154 L
s
.01923 .03846 m
.98077 .03846 L
s
.01923 .13462 m
.98077 .13462 L
s
.01923 .23077 m
.98077 .23077 L
s
.01923 .32692 m
.98077 .32692 L
s
.01923 .42308 m
.98077 .42308 L
s
.01923 .51923 m
.98077 .51923 L
s
.01923 .61538 m
.98077 .61538 L
s
.01923 .71154 m
.98077 .71154 L
s
.01923 .80769 m
.98077 .80769 L
s
.01923 .90385 m
.98077 .90385 L
s
.01923 1 m
.98077 1 L
s
.01923 1.09615 m
.98077 1.09615 L
s
.01923 1.19231 m
.98077 1.19231 L
s
.01923 1.28846 m
.98077 1.28846 L
s
.01923 1.38462 m
.98077 1.38462 L
s
.01923 1.48077 m
.98077 1.48077 L
s
.01923 1.57692 m
.98077 1.57692 L
s
.01923 1.67308 m
.98077 1.67308 L
s
.01923 1.76923 m
.98077 1.76923 L
s
.01923 1.86538 m
.98077 1.86538 L
s
.01923 1.96154 m
.98077 1.96154 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[5]:=",
  ImageSize->{181, 362},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0001JR000`400?l00000o`00003o]Oooool0
0;Gooooo002eooooo`00]Oooool00;Gooooo002eooooo`00]Oooool00;Gooooo0003ooooojkoIVIV
1?ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_oo
WSoooiho00kooiho00?oIVIVogconomloo/03_mloo/00omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0
O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_oo
WSoooiho00oooiho00?oIVIVoonN?oooWSl03_ooWSl00omVIVKoooooooooo`02ooooo`000oooool0
0omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVoonN?oooWSl03_ooWSl0
0omVIVKoO?okogcon`0>ogcon`03ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d0
0omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl0
0omVIVKooihooonN?`0>oonN?`03ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKooihooonN?`0>oonN?`03ofIVI_mloo_oO?ok
00koO?ok00?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKooCm6
00kooCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_ooWSoooiho
00kooiho00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
oihooonN?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?oIVIVogconomloo/03_mloo/00omVIVKo
P7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo
0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVoonN?oooWSl03_ooWSl00omVIVKo
ooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho
00?oIVIVoonN?oooWSl03_ooWSl00omVIVKoO?okogcon`0>ogcon`03ofIVI_n0O_goP7km00ooP7km
00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko0000
00?oIVIVoonN?oooWSl03oooWSl00omVIVKooihooonN?`0>oonN?`03ofIVI_oooooooooo00;ooooo
0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKooihooonN
?`0>oonN?`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1n
o@0>oh1no@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN
?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?oIVIVooooooooool00_ooool000?ooooo00?oIVIV
o`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?oIVIV
ogconomloo/03_mloo/00omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIV
oodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIV
oonN?oooWSl03_ooWSl00omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`00
0003ofIVI_ooWSoooiho00oooiho00?oIVIVoonN?oooWSl03_ooWSl00omVIVKoO?okogcon`0>ogco
n`03ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKooCm6oodoAP0>oodo
AP03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKooihooonN?`0>oonN
?`03ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVoonN?ooo
WSl03oooWSl00omVIVKooihooonN?`0>oonN?`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVoh1noOn0
O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l0
00003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?oIVIVoooooooo
ool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIV
I_ooWSoooiho00kooiho00?oIVIVogconomloo/03_mloo/00omVIVKoP7kmoh1no@0?oh1no@03ofIV
I_n0O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIV
I_ooWSoooiho00oooiho00?oIVIVoonN?oooWSl03_ooWSl00omVIVKoooooooooo`02ooooo`000ooo
ool00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVoonN?oooWSl03_oo
WSl00omVIVKoO?okogcon`0>ogcon`03ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0
O_d00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03ooo
WSl00omVIVKooihooonN?`0>oonN?`03ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKooihooonN?`0>oonN?`03ofIVI_mloo_o
O?ok00koO?ok00?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKo
oCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_ooWSoo
oiho00kooiho00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKooihooonN?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?oIVIVogconomloo/03_mloo/00omV
IVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omV
IVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVoonN?oooWSl03_ooWSl00omV
IVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oo
oiho00?oIVIVoonN?oooWSl03_ooWSl00omVIVKoO?okogcon`0>ogcon`03ofIVI_n0O_goP7km00oo
P7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko
000000?oIVIVoonN?oooWSl03oooWSl00omVIVKooihooonN?`0>oonN?`03ofIVI_oooooooooo00;o
oooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKooiho
oonN?`0>oonN?`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7km
oh1no@0>oh1no@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKooiho
oonN?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?oIVIVooooooooool00_ooool000?ooooo00?o
IVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_ooWSoooiho00kooiho00?o
IVIVogconomloo/03_mloo/00omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?o
IVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?o
IVIVoonN?oooWSl03_ooWSl00omVIVKoooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003oooo
o`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1n
o@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN
?`03ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVoodoA_om
?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVo`000?l0
00003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKooCm6oodoAP0>oodoAP03ofIV
I_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKo0000o`00000>o`000003ofIV
I_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0
O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l0
00000omVIVKooihooonN?`0?oonN?`03ofIVI_l0003o000000ko000000?oIVIVooooooooool00_oo
ool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_go
P7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_ooWSoo
oiho00oooiho00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omV
IVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl00omV
IVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKooCm600ko
oCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_l0003o000000ko
000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7km
oh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000
o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVo`000?l000003_l000000omVIVKooooo
ooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?o
IVIVoh1noOn0O_d03_n0O_d00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?o
IVIVoonN?oooWSl03oooWSl00omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003
ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>
oh1no@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?
oonN?`03ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVoodo
A_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVo`00
0?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKooCm6oodoAP0>oodoAP03
ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl00omVIVKo0000o`00000>o`000003
ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d0
3on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l00000
3_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_l0003o000000ko000000?oIVIVooooooooool0
0_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0
O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_oo
WSoooiho00oooiho00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d0
0omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVoonN?oooWSl03oooWSl0
0omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_om?dKooCm6
00kooCm600?oIVIVo`000?l000003_l000000omVIVKooihooonN?`0?oonN?`03ofIVI_l0003o0000
00ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
P7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVoodoA_om?dH03_om?dH00omVIVKo
0000o`00000>o`000003ofIVI_ooWSoooiho00oooiho00?oIVIVo`000?l000003_l000000omVIVKo
ooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02
ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/0
3_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0>ogcon`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oo
oooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool0
00?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
O?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooooooo
o`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003oooo
o`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVogconoml
oo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0>ogcon`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_oo
ool000?ooooo[_mVIVH4ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok
00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo
00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo
O?okogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok
00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogco
n`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
ogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
ooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogco
n`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02oooo
o`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconoml
oo/03_mloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIV
I_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_ml
oo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_o
O?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooo
oooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omV
IVKoO?okogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?o
oooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00ko
O?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?ok
ogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?
ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02
ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogco
nomloo/03_mloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooojko
IVIV1?ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000ooo
ool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00oo
O?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;o
oooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogco
nomloo/03omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooo
ooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`00
0oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok
00ooO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo
00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003
ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/0
3omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool0
0_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
O?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
ooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo
0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconoml
oo/03omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooooooo
ool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000ooo
ool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVogconomloo/03omloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKoO?okogcon`0?ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKoooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogco
n`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooooooo
o`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIV
ogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003oooo
o`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogco
n`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03oml
oo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_oo
ool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_o
O?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?ok
ogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooo
ooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?o
IVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003
ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>
ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/0
3omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool0
0_ooool000?ooooo[_mVIVH4ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooo
oooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKooCm6oodoAP0>oodoAP03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?o
oooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600ko
oCm600?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKooCm6
oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_om?dKooCm600kooCm600?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_om?dKooCm600kooCm600?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02
ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodo
A_om?dH03_om?dH00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03
ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIVoodoA_om?dH0
3_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oo
oooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKooCm6oodoAP0>oodoAP03ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool0
00?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm6
00kooCm600?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo
oCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_om?dKooCm600kooCm6
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooooooo
o`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
oodoA_om?dH03_om?dH00omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003oooo
ojkoIVIV1?ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`00
0oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH0
3_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo
00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
oCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm6
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
ooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02oooo
o`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om
?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooo
oooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?o
oooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600ko
oCm600?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooojkoIVIV1?ooool000?ooooo00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooooooo
ool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000ooo
ool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om
?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;o
oooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooCm6
oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH03_om?dH00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooo
ooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`00
0oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVoodoA_om?dH0
3_om?dH00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooCm6oodoAP0>oodoAP03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_om?dKooCm600kooCm600?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKoooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooo
ooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003
ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>
ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool0
0_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_om
O?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVooeloOomO?d03_omO?d0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
ooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo
0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooGcmooel
o@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooooooo
ool00_ooool000?ooooo[_mVIVH4ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVooeloOomO?d03_omO?d00omVIVKooGcmooelo@0>ooelo@03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oo
oooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKooGcmooelo@0>ooelo@03ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool0
00?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_omO?gooGcm
00kooGcm00?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVooeloOomO?d03_omO?d00omVIVKo
oGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>ooelo@03ofIVI_omO?gooGcm00kooGcm
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooooooo
o`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
ooeloOomO?d03_omO?d00omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003oooo
o`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>ooel
o@03ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVooeloOom
O?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVooeloOomO?d03_omO?d00omVIVKooGcmooelo@0>ooelo@03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKooGcmooelo@0>ooelo@03ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_oo
ool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_omO?go
oGcm00kooGcm00?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVooeloOomO?d03_omO?d00omV
IVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>ooelo@03ofIVI_omO?gooGcm00ko
oGcm00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVooeloOomO?d03_omO?d00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooo
ooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVooeloOomO?d03_omO?d00omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003
ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKooGcmooelo@0>
ooelo@03ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo[_mVIVH4oooo
o`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooeloOomO?d03_om
O?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooo
oooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?o
oooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooGcm
ooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_omO?gooGcm00kooGcm00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02
ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooeloOomO?d0
3_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooGcmooelo@0>ooelo@03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oo
oooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_omO?gooGcm00kooGcm00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool0
00?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVooeloOomO?d03_omO?d00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
oGcmooelo@0>ooelo@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooojkoIVIV1?ooool000?ooooo00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0>
ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooo
ooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`00
0oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_ml
oo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03_mloo/0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo
00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoO?okogco
n`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00koO?ok00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
ooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02oooo
o`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03_ml
oo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKoooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_mloo_oO?ok
00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
O?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
ooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok
00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo
0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogco
n`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIV
ogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogcon`0>ogco
n`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconoml
oo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooooooo
ool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIV
I_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000ooo
ool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_ml
oo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_mloo_o
O?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_mloo/00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_mloo_oO?ok00oo
O?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;o
oooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogconomloo/03omloo/00omVIVKoO?ok
ogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03ofIVI_mloo_oO?ok00koO?ok00?o
IVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_mloo_oO?ok00ooO?ok00?oIVIVogconomloo/03_mloo/00omVIVKoO?okogcon`0>
ogcon`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVogco
nomloo/03omloo/00omVIVKoO?okogcon`0>ogcon`03ofIVI_mloo_oO?ok00koO?ok00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooo
ooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoO?okogcon`0?ogcon`03
ofIVI_mloo_oO?ok00koO?ok00?oIVIVogconomloo/03_mloo/00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`00
0ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0
O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_oo
ool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_go
P7km00koP7km00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7km
oh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooo
ooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?o
IVIVoh1noOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003
ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>
oh1no@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d0
3on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool0
0_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0
O_goP7km00koP7km00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
P7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
ooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km
00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo
0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1n
o@0>oh1no@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo[_mVIVH4
ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1n
oOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_n0
O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oo
oooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d0
0omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool0
00?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km
00koP7km00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1n
o@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKooooooooo
o`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIV
oh1noOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003oooo
o`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0O_d00omVIVKoP7kmoh1no@0>oh1n
o@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_goP7km00koP7km00?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVoh1noOn0O_d03on0
O_d00omVIVKoP7kmoh1no@0>oh1no@03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_oo
ool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKoP7kmoh1no@0?oh1no@03ofIVI_n0O_go
P7km00koP7km00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_n0O_goP7km00ooP7km00?oIVIVoh1noOn0O_d03_n0O_d00omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooojkoIVIV1?ooool000?ooooo
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
ooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02oooo
o`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooo
oooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?o
oooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02
ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKoooooooooo`02ooooo`000ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;o
oooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?o
IVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?o
IVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>
o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>
o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`00
0?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooo
ooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003
ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003
ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`00
0oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo
00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
ooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02oooo
o`000ooooon^ofIVIPCooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l00000
3_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool0
0_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0
003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0
003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool0
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l00000
0omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol00000
0omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o0000
00ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o0000
00ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo
0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo
ooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo0000
00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko0000
00?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo
0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00
000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00
000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIV
o`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIV
o`000?l000003_l000000omVIVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`00
0003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`00
0003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l0
00003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVoooooooo
ool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIV
I_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIV
I_l0003o000000oo000000?oIVIVo`000?l000003_l000000omVIVKoooooooooo`02ooooo`000ooo
ool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l0
00000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol0
00000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;ooooo0003ooooo`03ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_l0003o
000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000?o`000003ofIVI_l0003o
000000ko000000?oIVIVooooooooool00_ooool000?ooooo00?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omV
IVKo0000o`00000>o`000003ofIVI_l0003o000000oo000000?oIVIVo`000?l000003_l000000omV
IVKoooooooooo`02ooooo`000oooool00omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000oo
000000?oIVIVo`000?l000003_l000000omVIVKo0000o`00000>o`000003ofIVI_l0003o000000ko
000000?oIVIVo`000?l000003ol000000omVIVKo0000o`00000>o`000003ofIVI_oooooooooo00;o
oooo0003ooooo`03ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003ol000000omVIVKo0000
o`00000>o`000003ofIVI_l0003o000000ko000000?oIVIVo`000?l000003_l000000omVIVKo0000
o`00000?o`000003ofIVI_l0003o000000ko000000?oIVIVooooooooool00_ooool000?ooooo[_mV
IVH4ooooo`00]Oooool00;Gooooo002eooooo`00]Oooool00;Gooooo002eooooo`00]Oooool00001
\
\>"],
  ImageRangeCache->{{{0, 180}, {361, 0}} -> {-0.200052, -0.428993, 0.0577784, \
0.0577784}},
  CellTags->"Tetris"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  DensityGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[5]=",
  CellTags->"Tetris"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Upcoming Features", "Subsection",
  CellTags->"Tetris"],

Cell["Partial list of upcoming features", "Text",
  CellTags->"Tetris"],

Cell["\<\
Wizard for choosing game dimensions, changing key commands and \
colors of pieces, and more.\
\>", "BulletedList",
  CellTags->"Tetris"],

Cell["\<\
Simulation mode for running without GUI, letting the code pick the \
\[OpenCurlyQuote]best\[CloseCurlyQuote] move.\
\>", "BulletedList",
  CellTags->"Tetris"],

Cell["Add custom pieces to game", "BulletedList",
  CellTags->"Tetris"],

Cell[TextData[{
  "Please send comments/questions to Andy Hunt (",
  ButtonBox["andy@wolfram.com",
    ButtonData:>{
      URL[ "mailto:andy@wolfram.com"], None},
    ButtonStyle->"Hyperlink"],
  ")."
}], "Text",
  CellTags->"Tetris"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Credits", "Subsection",
  CellTags->"Tetris"],

Cell[TextData[{
  "This has been, and continues to be a fun project. It wouldn't have been \
possible without the assistance of Jeff Adams (author of GUIKit (",
  ButtonBox["http://www.wolfram.com/solutions/mathlink/guikit/",
    ButtonData:>{
      URL[ "http://www.wolfram.com/solutions/mathlink/guikit/"], None},
    ButtonStyle->"Hyperlink"],
  ")."
}], "Text",
  CellTags->"Tetris"],

Cell[TextData[{
  "\[LongDash]Andy Hunt (",
  ButtonBox["andy@wolfram.com",
    ButtonData:>{
      URL[ "mailto:andy@wolfram.com"], None},
    ButtonStyle->"Hyperlink"],
  "),  February 29, 2004."
}], "Text",
  CellTags->"Tetris"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Macintosh",
ScreenRectangle->{{33, 1280}, {0, 938}},
WindowSize->{693, 591},
WindowMargins->{{1, Automatic}, {Automatic, 1}},
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Tetris"->{
    Cell[1776, 53, 58, 1, 61, "Section",
      CellTags->"Tetris"],
    Cell[1837, 56, 438, 12, 40, "Text",
      CellTags->"Tetris"],
    Cell[2278, 70, 234, 8, 24, "Text",
      CellTags->"Tetris"],
    Cell[2537, 82, 58, 1, 36, "Subsection",
      CellTags->"Tetris"],
    Cell[2598, 85, 60, 1, 40, "MathCaption",
      CellTags->"Tetris"],
    Cell[2661, 88, 99, 3, 26, "Input",
      CellTags->"Tetris"],
    Cell[2763, 93, 54, 1, 40, "MathCaption",
      CellTags->"Tetris"],
    Cell[2820, 96, 92, 3, 26, "Input",
      CellTags->"Tetris"],
    Cell[2915, 101, 58, 1, 40, "MathCaption",
      CellTags->"Tetris"],
    Cell[2976, 104, 98, 3, 26, "Input",
      CellTags->"Tetris"],
    Cell[3077, 109, 67, 1, 24, "IndentedText",
      CellTags->"Tetris"],
    Cell[3169, 114, 105, 3, 26, "Input",
      CellTags->"Tetris"],
    Cell[3277, 119, 115141, 1426, 383, 114974, 1420, "GraphicsData", 
    "Bitmap", "Graphics",
      Evaluatable->False,
      CellTags->"Tetris"],
    Cell[118467, 1551, 54, 1, 36, "Subsection",
      CellTags->"Tetris"],
    Cell[118524, 1554, 78, 1, 40, "MathCaption",
      CellTags->"Tetris"],
    Cell[118627, 1559, 103, 3, 26, "Input",
      CellTags->"Tetris"],
    Cell[118733, 1564, 104035, 1421, 368, 3146, 169, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"Tetris"],
    Cell[222771, 2987, 183, 5, 25, "Output",
      CellTags->"Tetris"],
    Cell[223003, 2998, 61, 1, 36, "Subsection",
      CellTags->"Tetris"],
    Cell[223067, 3001, 71, 1, 24, "Text",
      CellTags->"Tetris"],
    Cell[223141, 3004, 146, 4, 26, "BulletedList",
      CellTags->"Tetris"],
    Cell[223290, 3010, 168, 4, 26, "BulletedList",
      CellTags->"Tetris"],
    Cell[223461, 3016, 71, 1, 26, "BulletedList",
      CellTags->"Tetris"],
    Cell[223535, 3019, 234, 8, 24, "Text",
      CellTags->"Tetris"],
    Cell[223806, 3032, 51, 1, 36, "Subsection",
      CellTags->"Tetris"],
    Cell[223860, 3035, 387, 9, 40, "Text",
      CellTags->"Tetris"],
    Cell[224250, 3046, 231, 8, 24, "Text",
      CellTags->"Tetris"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Tetris", 225130, 3074}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 58, 1, 61, "Section",
  CellTags->"Tetris"],
Cell[1837, 56, 438, 12, 40, "Text",
  CellTags->"Tetris"],
Cell[2278, 70, 234, 8, 24, "Text",
  CellTags->"Tetris"],

Cell[CellGroupData[{
Cell[2537, 82, 58, 1, 36, "Subsection",
  CellTags->"Tetris"],
Cell[2598, 85, 60, 1, 40, "MathCaption",
  CellTags->"Tetris"],
Cell[2661, 88, 99, 3, 26, "Input",
  CellTags->"Tetris"],
Cell[2763, 93, 54, 1, 40, "MathCaption",
  CellTags->"Tetris"],
Cell[2820, 96, 92, 3, 26, "Input",
  CellTags->"Tetris"],
Cell[2915, 101, 58, 1, 40, "MathCaption",
  CellTags->"Tetris"],
Cell[2976, 104, 98, 3, 26, "Input",
  CellTags->"Tetris"],
Cell[3077, 109, 67, 1, 24, "IndentedText",
  CellTags->"Tetris"],

Cell[CellGroupData[{
Cell[3169, 114, 105, 3, 26, "Input",
  CellTags->"Tetris"],
Cell[3277, 119, 115141, 1426, 383, 114974, 1420, "GraphicsData", "Bitmap", \
"Graphics",
  Evaluatable->False,
  CellTags->"Tetris"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[118467, 1551, 54, 1, 36, "Subsection",
  CellTags->"Tetris"],
Cell[118524, 1554, 78, 1, 40, "MathCaption",
  CellTags->"Tetris"],

Cell[CellGroupData[{
Cell[118627, 1559, 103, 3, 26, "Input",
  CellTags->"Tetris"],
Cell[118733, 1564, 104035, 1421, 368, 3146, 169, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"Tetris"],
Cell[222771, 2987, 183, 5, 25, "Output",
  CellTags->"Tetris"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[223003, 2998, 61, 1, 36, "Subsection",
  CellTags->"Tetris"],
Cell[223067, 3001, 71, 1, 24, "Text",
  CellTags->"Tetris"],
Cell[223141, 3004, 146, 4, 26, "BulletedList",
  CellTags->"Tetris"],
Cell[223290, 3010, 168, 4, 26, "BulletedList",
  CellTags->"Tetris"],
Cell[223461, 3016, 71, 1, 26, "BulletedList",
  CellTags->"Tetris"],
Cell[223535, 3019, 234, 8, 24, "Text",
  CellTags->"Tetris"]
}, Open  ]],

Cell[CellGroupData[{
Cell[223806, 3032, 51, 1, 36, "Subsection",
  CellTags->"Tetris"],
Cell[223860, 3035, 387, 9, 40, "Text",
  CellTags->"Tetris"],
Cell[224250, 3046, 231, 8, 24, "Text",
  CellTags->"Tetris"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

